# encoding: utf-8

"""
lxml custom element classes for legend-related XML elements.
"""

from __future__ import absolute_import, print_function, unicode_literals

from ...enum.chart import XL_LEGEND_POSITION
from ..text import CT_TextBody
from ..xmlchemy import BaseOxmlElement, OptionalAttribute, ZeroOrOne


class CT_Legend(BaseOxmlElement):
    """
    ``<c:legend>`` custom element class
    """

    _tag_seq = (
        "c:legendPos",
        "c:legendEntry",
        "c:layout",
        "c:overlay",
        "c:spPr",
        "c:txPr",
        "c:extLst",
    )
    legendPos = ZeroOrOne("c:legendPos", successors=_tag_seq[1:])
    layout = ZeroOrOne("c:layout", successors=_tag_seq[3:])
    overlay = ZeroOrOne("c:overlay", successors=_tag_seq[4:])
    txPr = ZeroOrOne("c:txPr", successors=_tag_seq[6:])
    del _tag_seq

    @property
    def defRPr(self):
        """
        `./c:txPr/a:p/a:pPr/a:defRPr` great-great-grandchild element, added
        with its ancestors if not present.
        """
        txPr = self.get_or_add_txPr()
        defRPr = txPr.defRPr
        return defRPr

    @property
    def horz_offset(self):
        """
        The float value in ./c:layout/c:manualLayout/c:x when
        ./c:layout/c:manualLayout/c:xMode@val == "factor". 0.0 if that
        XPath expression has no match.
        """
        layout = self.layout
        if layout is None:
            return 0.0
        return layout.horz_offset

    @horz_offset.setter
    def horz_offset(self, offset):
        """
        Set the value of ./c:layout/c:manualLayout/c:x@val to *offset* and
        ./c:layout/c:manualLayout/c:xMode@val to "factor". Remove
        ./c:layout/c:manualLayout if *offset* == 0.
        """
        layout = self.get_or_add_layout()
        layout.horz_offset = offset

    def _new_txPr(self):
        return CT_TextBody.new_txPr()


class CT_LegendPos(BaseOxmlElement):
    """
    ``<c:legendPos>`` element specifying position of legend with respect to
    chart as a member of ST_LegendPos.
    """

    val = OptionalAttribute("val", XL_LEGEND_POSITION, default=XL_LEGEND_POSITION.RIGHT)



# encoding: utf-8

"""
Enumerations that describe click action settings
"""

from __future__ import absolute_import

from .base import alias, Enumeration, EnumMember


@alias("PP_ACTION")
class PP_ACTION_TYPE(Enumeration):
    """
    Specifies the type of a mouse action (click or hover action).

    Alias: ``PP_ACTION``

    Example::

        from pptx.enum.action import PP_ACTION

        assert shape.click_action.action == PP_ACTION.HYPERLINK
    """

    __ms_name__ = "PpActionType"

    __url__ = "https://msdn.microsoft.com/EN-US/library/office/ff744895.aspx"

    __members__ = (
        EnumMember("END_SHOW", 6, "Slide show ends."),
        EnumMember("FIRST_SLIDE", 3, "Returns to the first slide."),
        EnumMember("HYPERLINK", 7, "Hyperlink."),
        EnumMember("LAST_SLIDE", 4, "Moves to the last slide."),
        EnumMember("LAST_SLIDE_VIEWED", 5, "Moves to the last slide viewed."),
        EnumMember("NAMED_SLIDE", 101, "Moves to slide specified by slide number."),
        EnumMember("NAMED_SLIDE_SHOW", 10, "Runs the slideshow."),
        EnumMember("NEXT_SLIDE", 1, "Moves to the next slide."),
        EnumMember("NONE", 0, "No action is performed."),
        EnumMember("OPEN_FILE", 102, "Opens the specified file."),
        EnumMember("OLE_VERB", 11, "OLE Verb."),
        EnumMember("PLAY", 12, "Begins the slideshow."),
        EnumMember("PREVIOUS_SLIDE", 2, "Moves to the previous slide."),
        EnumMember("RUN_MACRO", 8, "Runs a macro."),
        EnumMember("RUN_PROGRAM", 9, "Runs a program."),
    )



# encoding: utf-8

"""
Test suite for pptx.chart.plot module
"""

from __future__ import absolute_import, print_function

import pytest

from pptx.chart.category import Categories
from pptx.chart.chart import Chart
from pptx.chart.plot import (
    _BasePlot,
    AreaPlot,
    Area3DPlot,
    BarPlot,
    BubblePlot,
    DataLabels,
    DoughnutPlot,
    LinePlot,
    PiePlot,
    PlotFactory,
    PlotTypeInspector,
    RadarPlot,
    XyPlot,
)
from pptx.chart.series import SeriesCollection
from pptx.enum.chart import XL_CHART_TYPE as XL

from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock


class Describe_BasePlot(object):
    def it_knows_which_chart_it_belongs_to(self, chart_fixture):
        plot, expected_value = chart_fixture
        assert plot.chart == expected_value

    def it_knows_whether_it_has_data_labels(self, has_data_labels_get_fixture):
        plot, expected_value = has_data_labels_get_fixture
        assert plot.has_data_labels == expected_value

    def it_can_change_whether_it_has_data_labels(self, has_data_labels_set_fixture):
        plot, new_value, expected_xml = has_data_labels_set_fixture
        plot.has_data_labels = new_value
        assert plot._element.xml == expected_xml

    def it_knows_whether_it_varies_color_by_category(
        self, vary_by_categories_get_fixture
    ):
        plot, expected_value = vary_by_categories_get_fixture
        assert plot.vary_by_categories == expected_value

    def it_can_change_whether_it_varies_color_by_category(
        self, vary_by_categories_set_fixture
    ):
        plot, new_value, expected_xml = vary_by_categories_set_fixture
        plot.vary_by_categories = new_value
        assert plot._element.xml == expected_xml

    def it_provides_access_to_its_categories(self, categories_fixture):
        plot, categories_, Categories_, xChart = categories_fixture
        categories = plot.categories
        Categories_.assert_called_once_with(xChart)
        assert categories is categories_

    def it_provides_access_to_the_data_labels(self, data_labels_fixture):
        plot, data_labels_, DataLabels_, dLbls = data_labels_fixture
        data_labels = plot.data_labels
        DataLabels_.assert_called_once_with(dLbls)
        assert data_labels is data_labels_

    def it_provides_access_to_its_series(self, series_fixture):
        plot, series_, SeriesCollection_, xChart = series_fixture
        series = plot.series
        SeriesCollection_.assert_called_once_with(xChart)
        assert series is series_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def categories_fixture(self, categories_, Categories_):
        xChart = element("c:barChart")
        plot = _BasePlot(xChart, None)
        return plot, categories_, Categories_, xChart

    @pytest.fixture
    def chart_fixture(self, chart_):
        plot = _BasePlot(None, chart_)
        expected_value = chart_
        return plot, expected_value

    @pytest.fixture
    def data_labels_fixture(self, DataLabels_, data_labels_):
        barChart = element("c:barChart/c:dLbls")
        dLbls = barChart[0]
        plot = _BasePlot(barChart, None)
        return plot, data_labels_, DataLabels_, dLbls

    @pytest.fixture(
        params=[
            ("c:barChart", False),
            ("c:barChart/c:dLbls", True),
            ("c:lineChart", False),
            ("c:lineChart/c:dLbls", True),
            ("c:pieChart", False),
            ("c:pieChart/c:dLbls", True),
        ]
    )
    def has_data_labels_get_fixture(self, request):
        xChart_cxml, expected_value = request.param
        plot = _BasePlot(element(xChart_cxml), None)
        return plot, expected_value

    @pytest.fixture(
        params=[
            ("c:barChart", True, "c:barChart/c:dLbls/+"),
            ("c:barChart/c:dLbls", True, "c:barChart/c:dLbls"),
            ("c:barChart", False, "c:barChart"),
            ("c:barChart/c:dLbls", False, "c:barChart"),
            ("c:bubbleChart", True, "c:bubbleChart/c:dLbls/+"),
            ("c:bubbleChart/c:dLbls", False, "c:bubbleChart"),
            ("c:lineChart", True, "c:lineChart/c:dLbls/+"),
            ("c:lineChart/c:dLbls", False, "c:lineChart"),
            ("c:pieChart", True, "c:pieChart/c:dLbls/+"),
            ("c:pieChart/c:dLbls", False, "c:pieChart"),
        ]
    )
    def has_data_labels_set_fixture(self, request):
        xChart_cxml, new_value, expected_cxml = request.param
        # apply extended suffix to replace trailing '+' where present
        if expected_cxml.endswith("+"):
            expected_cxml = expected_cxml[:-1] + (
                "(c:showLegendKey{val=0},c:showVal{val=1},c:showCatName{val="
                "0},c:showSerName{val=0},c:showPercent{val=0},c:showBubbleSi"
                "ze{val=0},c:showLeaderLines{val=1})"
            )
        plot = PlotFactory(element(xChart_cxml), None)
        expected_xml = xml(expected_cxml)
        return plot, new_value, expected_xml

    @pytest.fixture
    def series_fixture(self, SeriesCollection_, series_collection_):
        xChart = element("c:barChart")
        plot = _BasePlot(xChart, None)
        return plot, series_collection_, SeriesCollection_, xChart

    @pytest.fixture(
        params=[
            ("c:barChart", True),
            ("c:lineChart/c:varyColors", True),
            ("c:pieChart/c:varyColors{val=0}", False),
        ]
    )
    def vary_by_categories_get_fixture(self, request):
        xChart_cxml, expected_value = request.param
        plot = _BasePlot(element(xChart_cxml), None)
        return plot, expected_value

    @pytest.fixture(
        params=[
            ("c:barChart", False, "c:barChart/c:varyColors{val=0}"),
            ("c:barChart", True, "c:barChart/c:varyColors"),
            ("c:lineChart/c:varyColors{val=0}", True, "c:lineChart/c:varyColors"),
            ("c:pieChart/c:varyColors{val=1}", False, "c:pieChart/c:varyColors{val=0}"),
        ]
    )
    def vary_by_categories_set_fixture(self, request):
        xChart_cxml, new_value, expected_xChart_cxml = request.param
        plot = _BasePlot(element(xChart_cxml), None)
        expected_xml = xml(expected_xChart_cxml)
        return plot, new_value, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Categories_(self, request, categories_):
        return class_mock(
            request, "pptx.chart.plot.Categories", return_value=categories_
        )

    @pytest.fixture
    def categories_(self, request):
        return instance_mock(request, Categories)

    @pytest.fixture
    def chart_(self, request):
        return instance_mock(request, Chart)

    @pytest.fixture
    def DataLabels_(self, request, data_labels_):
        return class_mock(
            request, "pptx.chart.plot.DataLabels", return_value=data_labels_
        )

    @pytest.fixture
    def data_labels_(self, request):
        return instance_mock(request, DataLabels)

    @pytest.fixture
    def SeriesCollection_(self, request, series_collection_):
        return class_mock(
            request, "pptx.chart.plot.SeriesCollection", return_value=series_collection_
        )

    @pytest.fixture
    def series_collection_(self, request):
        return instance_mock(request, SeriesCollection)


class DescribeBarPlot(object):
    def it_knows_its_gap_width(self, gap_width_get_fixture):
        bar_plot, expected_value = gap_width_get_fixture
        assert bar_plot.gap_width == expected_value

    def it_can_change_its_gap_width(self, gap_width_set_fixture):
        bar_plot, new_value, expected_xml = gap_width_set_fixture
        bar_plot.gap_width = new_value
        assert bar_plot._element.xml == expected_xml

    def it_knows_how_much_it_overlaps_the_adjacent_bar(self, overlap_get_fixture):
        bar_plot, expected_value = overlap_get_fixture
        assert bar_plot.overlap == expected_value

    def it_can_change_how_much_it_overlaps_the_adjacent_bar(self, overlap_set_fixture):
        bar_plot, new_value, expected_xml = overlap_set_fixture
        bar_plot.overlap = new_value
        assert bar_plot._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:barChart", 150),
            ("c:barChart/c:gapWidth", 150),
            ("c:barChart/c:gapWidth{val=175}", 175),
            ("c:barChart/c:gapWidth{val=042%}", 42),
        ]
    )
    def gap_width_get_fixture(self, request):
        barChart_cxml, expected_value = request.param
        bar_plot = BarPlot(element(barChart_cxml), None)
        return bar_plot, expected_value

    @pytest.fixture(
        params=[
            ("c:barChart", 42, "c:barChart/c:gapWidth{val=42}"),
            ("c:barChart", 150, "c:barChart/c:gapWidth"),
            ("c:barChart/c:gapWidth{val=200%}", 300, "c:barChart/c:gapWidth{val=300}"),
        ]
    )
    def gap_width_set_fixture(self, request):
        barChart_cxml, new_value, expected_barChart_cxml = request.param
        bar_plot = BarPlot(element(barChart_cxml), None)
        expected_xml = xml(expected_barChart_cxml)
        return bar_plot, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:barChart", 0),
            ("c:barChart/c:overlap", 0),
            ("c:barChart/c:overlap{val=42}", 42),
            ("c:barChart/c:overlap{val=-42}", -42),
            ("c:barChart/c:overlap{val=-042%}", -42),
        ]
    )
    def overlap_get_fixture(self, request):
        barChart_cxml, expected_value = request.param
        bar_plot = BarPlot(element(barChart_cxml), None)
        return bar_plot, expected_value

    @pytest.fixture(
        params=[
            ("c:barChart", 42, "c:barChart/c:overlap{val=42}"),
            ("c:barChart/c:overlap{val=42}", 24, "c:barChart/c:overlap{val=24}"),
            ("c:barChart/c:overlap{val=42}", -5, "c:barChart/c:overlap{val=-5}"),
            ("c:barChart/c:overlap{val=42}", 0, "c:barChart"),
        ]
    )
    def overlap_set_fixture(self, request):
        barChart_cxml, new_value, expected_barChart_cxml = request.param
        bar_plot = BarPlot(element(barChart_cxml), None)
        expected_xml = xml(expected_barChart_cxml)
        return bar_plot, new_value, expected_xml


class DescribeBubblePlot(object):
    def it_knows_its_bubble_scale(self, bubble_scale_get_fixture):
        bubble_plot, expected_value = bubble_scale_get_fixture
        assert bubble_plot.bubble_scale == expected_value

    def it_can_change_its_bubble_scale(self, bubble_scale_set_fixture):
        bubble_plot, new_value, expected_xml = bubble_scale_set_fixture
        bubble_plot.bubble_scale = new_value
        assert bubble_plot._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:bubbleChart", 100),
            ("c:bubbleChart/c:bubbleScale", 100),
            ("c:bubbleChart/c:bubbleScale{val=175}", 175),
            ("c:bubbleChart/c:bubbleScale{val=070%}", 70),
        ]
    )
    def bubble_scale_get_fixture(self, request):
        bubbleChart_cxml, expected_value = request.param
        bubble_plot = BubblePlot(element(bubbleChart_cxml), None)
        return bubble_plot, expected_value

    @pytest.fixture(
        params=[
            ("c:bubbleChart", 42, "c:bubbleChart/c:bubbleScale{val=42}"),
            (
                "c:bubbleChart/c:bubbleScale{val=042%}",
                150,
                "c:bubbleChart/c:bubbleScale{val=150}",
            ),
            ("c:bubbleChart/c:bubbleScale{val=150}", None, "c:bubbleChart"),
            (
                "c:bubbleChart/c:bubbleScale{val=150}",
                100,
                "c:bubbleChart/c:bubbleScale",
            ),
            ("c:bubbleChart", None, "c:bubbleChart"),
            ("c:bubbleChart", 100, "c:bubbleChart/c:bubbleScale"),
        ]
    )
    def bubble_scale_set_fixture(self, request):
        bubbleChart_cxml, new_value, expected_cxml = request.param
        bubble_plot = BubblePlot(element(bubbleChart_cxml), None)
        expected_xml = xml(expected_cxml)
        return bubble_plot, new_value, expected_xml


class DescribePlotFactory(object):
    def it_contructs_a_plot_object_from_a_plot_element(self, call_fixture):
        xChart, chart_, PlotClass_, plot_ = call_fixture
        plot = PlotFactory(xChart, chart_)
        PlotClass_.assert_called_once_with(xChart, chart_)
        assert plot is plot_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:areaChart", AreaPlot),
            ("c:area3DChart", Area3DPlot),
            ("c:barChart", BarPlot),
            ("c:bubbleChart", BubblePlot),
            ("c:doughnutChart", DoughnutPlot),
            ("c:lineChart", LinePlot),
            ("c:pieChart", PiePlot),
            ("c:radarChart", RadarPlot),
            ("c:scatterChart", XyPlot),
        ]
    )
    def call_fixture(self, request, chart_):
        xChart_cxml, PlotCls = request.param
        plot_ = instance_mock(request, PlotCls, name="plot_")
        class_spec = "pptx.chart.plot.%s" % PlotCls.__name__
        PlotClass_ = class_mock(request, class_spec, return_value=plot_)
        xChart = element(xChart_cxml)
        return xChart, chart_, PlotClass_, plot_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_(self, request):
        return instance_mock(request, Chart)


class DescribePlotTypeInspector(object):
    def it_can_determine_the_chart_type_of_a_plot(self, chart_type_fixture):
        plot, expected_chart_type = chart_type_fixture
        chart_type = PlotTypeInspector.chart_type(plot)
        assert chart_type is expected_chart_type

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:areaChart", XL.AREA),
            ("c:areaChart/c:grouping", XL.AREA),
            ("c:areaChart/c:grouping{val=standard}", XL.AREA),
            ("c:areaChart/c:grouping{val=stacked}", XL.AREA_STACKED),
            ("c:areaChart/c:grouping{val=percentStacked}", XL.AREA_STACKED_100),
            ("c:area3DChart", XL.THREE_D_AREA),
            ("c:area3DChart/c:grouping", XL.THREE_D_AREA),
            ("c:area3DChart/c:grouping{val=standard}", XL.THREE_D_AREA),
            ("c:area3DChart/c:grouping{val=stacked}", XL.THREE_D_AREA_STACKED),
            (
                "c:area3DChart/c:grouping{val=percentStacked}",
                XL.THREE_D_AREA_STACKED_100,
            ),
            ("c:barChart/c:barDir", XL.COLUMN_CLUSTERED),
            ("c:barChart/c:barDir{val=col}", XL.COLUMN_CLUSTERED),
            ("c:barChart/(c:barDir{val=col},c:grouping)", XL.COLUMN_CLUSTERED),
            (
                "c:barChart/(c:barDir{val=col},c:grouping{val=clustered})",
                XL.COLUMN_CLUSTERED,
            ),
            (
                "c:barChart/(c:barDir{val=col},c:grouping{val=stacked})",
                XL.COLUMN_STACKED,
            ),
            (
                "c:barChart/(c:barDir{val=col},c:grouping{val=percentStacked})",
                XL.COLUMN_STACKED_100,
            ),
            ("c:barChart/c:barDir{val=bar}", XL.BAR_CLUSTERED),
            ("c:barChart/(c:barDir{val=bar},c:grouping)", XL.BAR_CLUSTERED),
            (
                "c:barChart/(c:barDir{val=bar},c:grouping{val=clustered})",
                XL.BAR_CLUSTERED,
            ),
            ("c:barChart/(c:barDir{val=bar},c:grouping{val=stacked})", XL.BAR_STACKED),
            (
                "c:barChart/(c:barDir{val=bar},c:grouping{val=percentStacked})",
                XL.BAR_STACKED_100,
            ),
            ("c:doughnutChart", XL.DOUGHNUT),
            ("c:doughnutChart/c:ser/c:explosion{val=25}", XL.DOUGHNUT_EXPLODED),
            ("c:lineChart", XL.LINE_MARKERS),
            ("c:lineChart/c:grouping", XL.LINE_MARKERS),
            ("c:lineChart/c:grouping{val=standard}", XL.LINE_MARKERS),
            ("c:lineChart/c:grouping{val=stacked}", XL.LINE_MARKERS_STACKED),
            ("c:lineChart/c:grouping{val=percentStacked}", XL.LINE_MARKERS_STACKED_100),
            ("c:lineChart/c:ser/c:marker/c:symbol{val=none}", XL.LINE),
            (
                "c:lineChart/(c:grouping{val=stacked},c:ser/c:marker/c:symbol{val=n"
                "one})",
                XL.LINE_STACKED,
            ),
            (
                "c:lineChart/(c:grouping{val=percentStacked},c:ser/c:marker/c:symbo"
                "l{val=none})",
                XL.LINE_STACKED_100,
            ),
            ("c:pieChart", XL.PIE),
            ("c:pieChart/c:ser/c:explosion{val=25}", XL.PIE_EXPLODED),
            ("c:scatterChart/c:scatterStyle", XL.XY_SCATTER),
            (
                "c:scatterChart/(c:scatterStyle{val=lineMarker},c:ser/c:spPr/a:ln/a"
                ":noFill)",
                XL.XY_SCATTER,
            ),
            ("c:scatterChart/c:scatterStyle{val=lineMarker}", XL.XY_SCATTER_LINES),
            (
                "c:scatterChart/(c:scatterStyle{val=lineMarker},c:ser/c:marker/c:sy"
                "mbol{val=none})",
                XL.XY_SCATTER_LINES_NO_MARKERS,
            ),
            (
                "c:scatterChart/(c:scatterStyle{val=lineMarker},c:ser/c:marker/c:sy"
                "mbol{val=diamond})",
                XL.XY_SCATTER_LINES,
            ),
            ("c:scatterChart/c:scatterStyle{val=smoothMarker}", XL.XY_SCATTER_SMOOTH),
            (
                "c:scatterChart/(c:scatterStyle{val=smoothMarker},c:ser/c:marker/c:"
                "symbol{val=none})",
                XL.XY_SCATTER_SMOOTH_NO_MARKERS,
            ),
            ("c:bubbleChart", XL.BUBBLE),
            ("c:bubbleChart/c:ser", XL.BUBBLE),
            ("c:bubbleChart/c:ser/c:bubble3D", XL.BUBBLE_THREE_D_EFFECT),
            ("c:bubbleChart/c:ser/c:bubble3D{val=0}", XL.BUBBLE),
            ("c:bubbleChart/c:ser/c:bubble3D{val=1}", XL.BUBBLE_THREE_D_EFFECT),
            ("c:radarChart/c:radarStyle", XL.RADAR),
            ("c:radarChart/c:radarStyle{val=marker}", XL.RADAR_MARKERS),
            ("c:radarChart/c:radarStyle{val=filled}", XL.RADAR_FILLED),
            (
                "c:radarChart/(c:radarStyle{val=marker},c:ser/c:marker/c:symbol{val"
                "=none})",
                XL.RADAR,
            ),
        ]
    )
    def chart_type_fixture(self, request):
        xChart_cxml, expected_chart_type = request.param
        plot = PlotFactory(element(xChart_cxml), None)
        return plot, expected_chart_type



# encoding: utf-8

"""
Test suite for pptx.chart.xlsx module
"""

from __future__ import absolute_import, print_function

import pytest

from xlsxwriter import Workbook
from xlsxwriter.worksheet import Worksheet

from pptx.chart.data import (
    BubbleChartData,
    Categories,
    CategoryChartData,
    CategorySeriesData,
    XyChartData,
)
from pptx.chart.xlsx import (
    _BaseWorkbookWriter,
    BubbleWorkbookWriter,
    CategoryWorkbookWriter,
    XyWorkbookWriter,
)
from pptx.compat import BytesIO

from ..unitutil.mock import ANY, call, class_mock, instance_mock, method_mock


class Describe_BaseWorkbookWriter(object):
    def it_can_generate_a_chart_data_Excel_blob(self, xlsx_blob_fixture):
        workbook_writer, xlsx_file_, workbook_, worksheet_, xlsx_blob = (
            xlsx_blob_fixture
        )
        _xlsx_blob = workbook_writer.xlsx_blob

        workbook_writer._open_worksheet.assert_called_once_with(xlsx_file_)
        workbook_writer._populate_worksheet.assert_called_once_with(
            workbook_writer, workbook_, worksheet_
        )
        assert _xlsx_blob is xlsx_blob

    def it_can_open_a_worksheet_in_a_context(self, open_fixture):
        wb_writer, xlsx_file_, workbook_, worksheet_, Workbook_ = open_fixture

        with wb_writer._open_worksheet(xlsx_file_) as (workbook, worksheet):
            Workbook_.assert_called_once_with(xlsx_file_, {"in_memory": True})
            workbook_.add_worksheet.assert_called_once_with()
            assert workbook is workbook_
            assert worksheet is worksheet_
        workbook_.close.assert_called_once_with()

    def it_raises_on_no_override_of_populate(self, populate_fixture):
        workbook_writer = populate_fixture
        with pytest.raises(NotImplementedError):
            workbook_writer._populate_worksheet(None, None)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def open_fixture(self, xlsx_file_, workbook_, worksheet_, Workbook_):
        workbook_writer = _BaseWorkbookWriter(None)
        workbook_.add_worksheet.return_value = worksheet_
        return workbook_writer, xlsx_file_, workbook_, worksheet_, Workbook_

    @pytest.fixture
    def populate_fixture(self):
        workbook_writer = _BaseWorkbookWriter(None)
        return workbook_writer

    @pytest.fixture
    def xlsx_blob_fixture(
        self,
        request,
        xlsx_file_,
        workbook_,
        worksheet_,
        _populate_worksheet_,
        _open_worksheet_,
        BytesIO_,
    ):
        workbook_writer = _BaseWorkbookWriter(None)
        xlsx_blob = "fooblob"
        BytesIO_.return_value = xlsx_file_
        # to make context manager behavior work
        _open_worksheet_.return_value.__enter__.return_value = (workbook_, worksheet_)
        xlsx_file_.getvalue.return_value = xlsx_blob
        return (workbook_writer, xlsx_file_, workbook_, worksheet_, xlsx_blob)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def BytesIO_(self, request):
        return class_mock(request, "pptx.chart.xlsx.BytesIO")

    @pytest.fixture
    def _open_worksheet_(self, request):
        return method_mock(request, _BaseWorkbookWriter, "_open_worksheet")

    @pytest.fixture
    def _populate_worksheet_(self, request):
        return method_mock(
            request, _BaseWorkbookWriter, "_populate_worksheet", autospec=True
        )

    @pytest.fixture
    def Workbook_(self, request, workbook_):
        return class_mock(request, "pptx.chart.xlsx.Workbook", return_value=workbook_)

    @pytest.fixture
    def workbook_(self, request):
        return instance_mock(request, Workbook)

    @pytest.fixture
    def worksheet_(self, request):
        return instance_mock(request, Worksheet)

    @pytest.fixture
    def xlsx_file_(self, request):
        return instance_mock(request, BytesIO)


class DescribeCategoryWorkbookWriter(object):
    def it_knows_the_categories_range_ref(self, categories_ref_fixture):
        workbook_writer, expected_value = categories_ref_fixture
        assert workbook_writer.categories_ref == expected_value

    def it_raises_on_cat_ref_on_no_categories(self, cat_ref_raises_fixture):
        workbook_writer = cat_ref_raises_fixture
        with pytest.raises(ValueError):
            workbook_writer.categories_ref

    def it_knows_the_ref_for_a_series_name(self, ser_name_ref_fixture):
        workbook_writer, series_, expected_value = ser_name_ref_fixture
        assert workbook_writer.series_name_ref(series_) == expected_value

    def it_knows_the_values_range_ref(self, values_ref_fixture):
        workbook_writer, series_, expected_value = values_ref_fixture
        assert workbook_writer.values_ref(series_) == expected_value

    def it_can_populate_a_worksheet_with_chart_data(self, populate_fixture):
        workbook_writer, workbook_, worksheet_ = populate_fixture
        workbook_writer._populate_worksheet(workbook_, worksheet_)
        workbook_writer._write_categories.assert_called_once_with(
            workbook_writer, workbook_, worksheet_
        )
        workbook_writer._write_series.assert_called_once_with(
            workbook_writer, workbook_, worksheet_
        )

    def it_calculates_a_column_reference_to_help(self, col_ref_fixture):
        col_num, expected_value = col_ref_fixture
        column_reference = CategoryWorkbookWriter._column_reference(col_num)
        assert column_reference == expected_value

    def it_raises_on_col_number_out_of_range(self, raises_fixture):
        col_num = raises_fixture
        with pytest.raises(ValueError):
            CategoryWorkbookWriter._column_reference(col_num)

    def it_writes_categories_to_help(self, write_cats_fixture):
        workbook_writer, workbook_, worksheet_ = write_cats_fixture[:3]
        number_format, calls = write_cats_fixture[3:]

        workbook_writer._write_categories(workbook_, worksheet_)

        workbook_.add_format.assert_called_once_with({"num_format": number_format})
        assert workbook_writer._write_cat_column.call_args_list == calls

    def it_writes_a_category_column_to_help(self, write_cat_fixture):
        workbook_writer, worksheet_, col = write_cat_fixture[:3]
        level, num_format_, calls = write_cat_fixture[3:]
        workbook_writer._write_cat_column(worksheet_, col, level, num_format_)
        assert worksheet_.mock_calls == calls

    def it_writes_series_to_help(self, write_sers_fixture):
        workbook_writer, workbook_, worksheet_, calls = write_sers_fixture
        workbook_writer._write_series(workbook_, worksheet_)
        assert worksheet_.mock_calls == calls

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            (1, 1, "Sheet1!$A$2:$A$2"),
            (1, 3, "Sheet1!$A$2:$A$4"),
            (2, 4, "Sheet1!$A$2:$B$5"),
            (3, 8, "Sheet1!$A$2:$C$9"),
        ]
    )
    def categories_ref_fixture(self, request, chart_data_, categories_):
        depth, leaf_count, expected_value = request.param
        workbook_writer = CategoryWorkbookWriter(chart_data_)
        chart_data_.categories = categories_
        categories_.depth, categories_.leaf_count = depth, leaf_count
        return workbook_writer, expected_value

    @pytest.fixture
    def cat_ref_raises_fixture(self, request, chart_data_, categories_):
        workbook_writer = CategoryWorkbookWriter(chart_data_)
        chart_data_.categories = categories_
        categories_.depth = 0
        return workbook_writer

    @pytest.fixture(
        params=[
            (2, "B"),
            (26, "Z"),
            (27, "AA"),
            (52, "AZ"),
            (53, "BA"),
            (676, "YZ"),
            (677, "ZA"),
            (702, "ZZ"),
            (703, "AAA"),
            (728, "AAZ"),
            (729, "ABA"),
            (1378, "AZZ"),
            (1379, "BAA"),
            (16384, "XFD"),
        ]
    )
    def col_ref_fixture(self, request):
        column_number, expected_value = request.param
        return column_number, expected_value

    @pytest.fixture
    def populate_fixture(
        self, workbook_, worksheet_, _write_categories_, _write_series_
    ):
        workbook_writer = CategoryWorkbookWriter(None)
        return workbook_writer, workbook_, worksheet_

    @pytest.fixture(params=[0, -1, 16385, 30433])
    def raises_fixture(self, request):
        column_number = request.param
        return column_number

    @pytest.fixture(
        params=[
            (1, 0, "Sheet1!$B$1"),
            (1, 3, "Sheet1!$E$1"),
            (3, 0, "Sheet1!$D$1"),
            (3, 3, "Sheet1!$G$1"),
        ]
    )
    def ser_name_ref_fixture(self, request, series_data_, categories_):
        cat_depth, series_index, expected_value = request.param
        workbook_writer = CategoryWorkbookWriter(None)
        series_data_.categories = categories_
        categories_.depth = cat_depth
        series_data_.index = series_index
        return workbook_writer, series_data_, expected_value

    @pytest.fixture(
        params=[
            (1, 0, 3, "Sheet1!$B$2:$B$4"),
            (1, 1, 3, "Sheet1!$C$2:$C$4"),
            (2, 0, 5, "Sheet1!$C$2:$C$6"),
            (3, 2, 7, "Sheet1!$F$2:$F$8"),
        ]
    )
    def values_ref_fixture(self, request, series_data_, categories_):
        cat_depth, ser_idx, val_count, expected_value = request.param
        workbook_writer = CategoryWorkbookWriter(None)
        series_data_.categories = categories_
        categories_.depth = cat_depth
        series_data_.index = ser_idx
        series_data_.__len__.return_value = val_count
        return workbook_writer, series_data_, expected_value

    @pytest.fixture
    def write_cat_fixture(self, worksheet_):
        workbook_writer = CategoryWorkbookWriter(None)
        col, level = 6, ([0, "Foo"], [1, "Bar"])
        num_format = object()
        calls = [
            call.set_column(col, col, 10),
            call.write(1, col, "Foo", num_format),
            call.write(2, col, "Bar", num_format),
        ]
        return workbook_writer, worksheet_, col, level, num_format, calls

    @pytest.fixture(
        params=[
            [[[0, "Foo"], [1, "Bar"], [2, "Baz"]]],
            [[[0, "CA"], [1, "NV"], [2, "NY"], [3, "NJ"]], [[0, "WEST"], [2, "EAST"]]],
        ]
    )
    def write_cats_fixture(
        self,
        request,
        chart_data_,
        workbook_,
        worksheet_,
        categories_,
        _write_cat_column_,
    ):
        levels = request.param
        workbook_writer = CategoryWorkbookWriter(chart_data_)
        number_format = categories_.number_format = "$#0.#"
        num_format = workbook_.add_format.return_value
        _depth = len(levels)
        calls = [
            call(workbook_writer, worksheet_, _depth - idx - 1, level, num_format)
            for (idx, level) in enumerate(levels)
        ]
        chart_data_.categories = categories_
        categories_.depth = len(levels)
        categories_.levels = levels
        return workbook_writer, workbook_, worksheet_, number_format, calls

    @pytest.fixture
    def write_sers_fixture(
        self, request, chart_data_, workbook_, worksheet_, categories_
    ):
        workbook_writer = CategoryWorkbookWriter(chart_data_)
        num_format = workbook_.add_format.return_value
        calls = [call.write(0, 1, "S1"), call.write_column(1, 1, (42, 24), num_format)]
        ser = instance_mock(request, CategorySeriesData, values=(42, 24))
        ser.name = "S1"
        chart_data_.categories = categories_
        categories_.depth = 1
        chart_data_.__iter__.return_value = iter([ser])
        return workbook_writer, workbook_, worksheet_, calls

    # fixture components ---------------------------------------------

    @pytest.fixture
    def categories_(self, request):
        return instance_mock(request, Categories)

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, CategoryChartData)

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, CategorySeriesData)

    @pytest.fixture
    def workbook_(self, request):
        return instance_mock(request, Workbook)

    @pytest.fixture
    def worksheet_(self, request):
        return instance_mock(request, Worksheet)

    @pytest.fixture
    def _write_cat_column_(self, request):
        return method_mock(
            request, CategoryWorkbookWriter, "_write_cat_column", autospec=True
        )

    @pytest.fixture
    def _write_categories_(self, request):
        return method_mock(
            request, CategoryWorkbookWriter, "_write_categories", autospec=True
        )

    @pytest.fixture
    def _write_series_(self, request):
        return method_mock(
            request, CategoryWorkbookWriter, "_write_series", autospec=True
        )


class DescribeBubbleWorkbookWriter(object):
    def it_can_populate_a_worksheet_with_chart_data(self, populate_fixture):
        workbook_writer, workbook_, worksheet_, expected_calls = populate_fixture
        workbook_writer._populate_worksheet(workbook_, worksheet_)
        assert worksheet_.mock_calls == expected_calls

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def populate_fixture(self, workbook_, worksheet_):
        chart_data = BubbleChartData()
        series_1 = chart_data.add_series("Series 1")
        for pt in ((1, 1.1, 10), (2, 2.2, 20)):
            series_1.add_data_point(*pt)
        series_2 = chart_data.add_series("Series 2")
        for pt in ((3, 3.3, 30), (4, 4.4, 40)):
            series_2.add_data_point(*pt)

        workbook_writer = BubbleWorkbookWriter(chart_data)

        expected_calls = [
            call.write_column(1, 0, [1, 2], ANY),
            call.write(0, 1, "Series 1"),
            call.write_column(1, 1, [1.1, 2.2], ANY),
            call.write(0, 2, "Size"),
            call.write_column(1, 2, [10, 20], ANY),
            call.write_column(5, 0, [3, 4], ANY),
            call.write(4, 1, "Series 2"),
            call.write_column(5, 1, [3.3, 4.4], ANY),
            call.write(4, 2, "Size"),
            call.write_column(5, 2, [30, 40], ANY),
        ]
        return workbook_writer, workbook_, worksheet_, expected_calls

    # fixture components ---------------------------------------------

    @pytest.fixture
    def workbook_(self, request):
        return instance_mock(request, Workbook)

    @pytest.fixture
    def worksheet_(self, request):
        return instance_mock(request, Worksheet)


class DescribeXyWorkbookWriter(object):
    def it_can_generate_a_chart_data_Excel_blob(self, xlsx_blob_fixture):
        workbook_writer, _open_worksheet_, xlsx_file_ = xlsx_blob_fixture[:3]
        _populate_worksheet_, workbook_, worksheet_ = xlsx_blob_fixture[3:6]
        xlsx_blob_ = xlsx_blob_fixture[6]

        xlsx_blob = workbook_writer.xlsx_blob

        _open_worksheet_.assert_called_once_with(xlsx_file_)
        _populate_worksheet_.assert_called_once_with(workbook_, worksheet_)
        assert xlsx_blob is xlsx_blob_

    def it_can_populate_a_worksheet_with_chart_data(self, populate_fixture):
        workbook_writer, workbook_, worksheet_, expected_calls = populate_fixture
        workbook_writer._populate_worksheet(workbook_, worksheet_)
        assert worksheet_.mock_calls == expected_calls

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def populate_fixture(self, workbook_, worksheet_):
        chart_data = XyChartData()
        series_1 = chart_data.add_series("Series 1")
        for pt in ((1, 1.1), (2, 2.2)):
            series_1.add_data_point(*pt)
        series_2 = chart_data.add_series("Series 2")
        for pt in ((3, 3.3), (4, 4.4)):
            series_2.add_data_point(*pt)

        workbook_writer = XyWorkbookWriter(chart_data)

        expected_calls = [
            call.write_column(1, 0, [1, 2], ANY),
            call.write(0, 1, "Series 1"),
            call.write_column(1, 1, [1.1, 2.2], ANY),
            call.write_column(5, 0, [3, 4], ANY),
            call.write(4, 1, "Series 2"),
            call.write_column(5, 1, [3.3, 4.4], ANY),
        ]
        return workbook_writer, workbook_, worksheet_, expected_calls

    @pytest.fixture
    def xlsx_blob_fixture(
        self,
        request,
        xlsx_file_,
        BytesIO_,
        _open_worksheet_,
        workbook_,
        worksheet_,
        _populate_worksheet_,
        xlsx_blob_,
    ):
        workbook_writer = XyWorkbookWriter(None)
        return (
            workbook_writer,
            _open_worksheet_,
            xlsx_file_,
            _populate_worksheet_,
            workbook_,
            worksheet_,
            xlsx_blob_,
        )

    # fixture components ---------------------------------------------

    @pytest.fixture
    def BytesIO_(self, request, xlsx_file_):
        return class_mock(request, "pptx.chart.xlsx.BytesIO", return_value=xlsx_file_)

    @pytest.fixture
    def _open_worksheet_(self, request, workbook_, worksheet_):
        open_worksheet_ = method_mock(request, XyWorkbookWriter, "_open_worksheet")
        # to make context manager behavior work
        open_worksheet_.return_value.__enter__.return_value = (workbook_, worksheet_)
        return open_worksheet_

    @pytest.fixture
    def _populate_worksheet_(self, request):
        return method_mock(request, XyWorkbookWriter, "_populate_worksheet")

    @pytest.fixture
    def workbook_(self, request):
        return instance_mock(request, Workbook)

    @pytest.fixture
    def worksheet_(self, request):
        return instance_mock(request, Worksheet)

    @pytest.fixture
    def xlsx_blob_(self, request):
        return instance_mock(request, bytes)

    @pytest.fixture
    def xlsx_file_(self, request, xlsx_blob_):
        xlsx_file_ = instance_mock(request, BytesIO)
        xlsx_file_.getvalue.return_value = xlsx_blob_
        return xlsx_file_



# encoding: utf-8

"""
Custom element classes for presentation-related XML elements.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from .simpletypes import ST_SlideId, ST_SlideSizeCoordinate, XsdString
from .xmlchemy import BaseOxmlElement, RequiredAttribute, ZeroOrOne, ZeroOrMore


class CT_Presentation(BaseOxmlElement):
    """
    ``<p:presentation>`` element, root of the Presentation part stored as
    ``/ppt/presentation.xml``.
    """

    sldMasterIdLst = ZeroOrOne(
        "p:sldMasterIdLst",
        successors=(
            "p:notesMasterIdLst",
            "p:handoutMasterIdLst",
            "p:sldIdLst",
            "p:sldSz",
            "p:notesSz",
        ),
    )
    sldIdLst = ZeroOrOne("p:sldIdLst", successors=("p:sldSz", "p:notesSz"))
    sldSz = ZeroOrOne("p:sldSz", successors=("p:notesSz",))


class CT_SlideId(BaseOxmlElement):
    """
    ``<p:sldId>`` element, direct child of <p:sldIdLst> that contains an rId
    reference to a slide in the presentation.
    """

    id = RequiredAttribute("id", ST_SlideId)
    rId = RequiredAttribute("r:id", XsdString)


class CT_SlideIdList(BaseOxmlElement):
    """
    ``<p:sldIdLst>`` element, direct child of <p:presentation> that contains
    a list of the slide parts in the presentation.
    """

    sldId = ZeroOrMore("p:sldId")

    def add_sldId(self, rId):
        """
        Return a reference to a newly created <p:sldId> child element having
        its r:id attribute set to *rId*.
        """
        return self._add_sldId(id=self._next_id, rId=rId)

    @property
    def _next_id(self):
        """
        Return the next available slide ID as an int. Valid slide IDs start
        at 256. The next integer value greater than the max value in use is
        chosen, which minimizes that chance of reusing the id of a deleted
        slide.
        """
        id_str_lst = self.xpath("./p:sldId/@id")
        return max([255] + [int(id_str) for id_str in id_str_lst]) + 1


class CT_SlideMasterIdList(BaseOxmlElement):
    """
    ``<p:sldMasterIdLst>`` element, child of ``<p:presentation>`` containing
    references to the slide masters that belong to the presentation.
    """

    sldMasterId = ZeroOrMore("p:sldMasterId")


class CT_SlideMasterIdListEntry(BaseOxmlElement):
    """
    ``<p:sldMasterId>`` element, child of ``<p:sldMasterIdLst>`` containing
    a reference to a slide master.
    """

    rId = RequiredAttribute("r:id", XsdString)


class CT_SlideSize(BaseOxmlElement):
    """
    ``<p:sldSz>`` element, direct child of <p:presentation> that contains the
    width and height of slides in the presentation.
    """

    cx = RequiredAttribute("cx", ST_SlideSizeCoordinate)
    cy = RequiredAttribute("cy", ST_SlideSizeCoordinate)



# encoding: utf-8

"""GroupShape and related objects."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.dml.effect import ShadowFormat
from pptx.enum.shapes import MSO_SHAPE_TYPE
from pptx.shapes.base import BaseShape
from pptx.util import lazyproperty


class GroupShape(BaseShape):
    """A shape that acts as a container for other shapes."""

    @property
    def click_action(self):
        """Unconditionally raises `TypeError`.

        A group shape cannot have a click action or hover action.
        """
        raise TypeError("a group shape cannot have a click action")

    @property
    def has_text_frame(self):
        """Unconditionally |False|.

        A group shape does not have a textframe and cannot itself contain
        text. This does not impact the ability of shapes contained by the
        group to each have their own text.
        """
        return False

    @lazyproperty
    def shadow(self):
        """|ShadowFormat| object representing shadow effect for this group.

        A |ShadowFormat| object is always returned, even when no shadow is
        explicitly defined on this group shape (i.e. when the group inherits
        its shadow behavior).
        """
        return ShadowFormat(self._element.grpSpPr)

    @property
    def shape_type(self):
        """Member of :ref:`MsoShapeType` identifying the type of this shape.

        Unconditionally `MSO_SHAPE_TYPE.GROUP` in this case
        """
        return MSO_SHAPE_TYPE.GROUP

    @lazyproperty
    def shapes(self):
        """|GroupShapes| object for this group.

        The |GroupShapes| object provides access to the group's member shapes
        and provides methods for adding new ones.
        """
        from pptx.shapes.shapetree import GroupShapes

        return GroupShapes(self._element, self)



# encoding: utf-8

"""
Test suite for pptx.oxml.shapes.picture module.
"""

from __future__ import absolute_import, print_function, unicode_literals

import pytest

from pptx.oxml.ns import nsdecls
from pptx.oxml.shapes.picture import CT_Picture


class DescribeCT_Picture(object):
    def it_can_create_a_new_pic_element(self, pic_fixture):
        shape_id, name, desc, rId, x, y, cx, cy, expected_xml = pic_fixture
        pic = CT_Picture.new_pic(shape_id, name, desc, rId, x, y, cx, cy)
        assert pic.xml == expected_xml

    def it_can_create_a_new_video_pic_element(self, video_pic_fixture):
        shape_id, shape_name, video_rId, media_rId = video_pic_fixture[:4]
        poster_frame_rId, x, y, cx, cy, expected_xml = video_pic_fixture[4:]
        pic = CT_Picture.new_video_pic(
            shape_id, shape_name, video_rId, media_rId, poster_frame_rId, x, y, cx, cy
        )
        print(pic.xml)
        assert pic.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def pic_fixture(self):
        shape_id, name, desc, rId = 9, "Diam. > 1 mm", "desc", "rId1"
        x, y, cx, cy = 1, 2, 3, 4
        expected_xml = (
            '<p:pic %s>\n  <p:nvPicPr>\n    <p:cNvPr id="%d" name="%s" descr'
            '="%s"/>\n    <p:cNvPicPr>\n      <a:picLocks noChangeAspect="1"'
            "/>\n    </p:cNvPicPr>\n    <p:nvPr/>\n  </p:nvPicPr>\n  <p:blip"
            'Fill>\n    <a:blip r:embed="%s"/>\n    <a:stretch>\n      <a:fi'
            "llRect/>\n    </a:stretch>\n  </p:blipFill>\n  <p:spPr>\n    <a"
            ':xfrm>\n      <a:off x="%d" y="%d"/>\n      <a:ext cx="%d" cy="'
            '%d"/>\n    </a:xfrm>\n    <a:prstGeom prst="rect">\n      <a:av'
            "Lst/>\n    </a:prstGeom>\n  </p:spPr>\n</p:pic>\n"
            % (
                nsdecls("a", "p", "r"),
                shape_id,
                "Diam. &gt; 1 mm",
                desc,
                rId,
                x,
                y,
                cx,
                cy,
            )
        )
        return shape_id, name, desc, rId, x, y, cx, cy, expected_xml

    @pytest.fixture
    def video_pic_fixture(self):
        shape_id, shape_name = 42, "media.mp4"
        video_rId, media_rId, poster_frame_rId = "rId1", "rId2", "rId3"
        x, y, cx, cy = 1, 2, 3, 4
        expected_xml = (
            '<p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/200'
            '6/main" xmlns:p="http://schemas.openxmlformats.org/presentation'
            'ml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
            'Document/2006/relationships">\n  <p:nvPicPr>\n    <p:cNvPr id="'
            '42" name="media.mp4">\n      <a:hlinkClick r:id="" action="ppac'
            'tion://media"/>\n    </p:cNvPr>\n    <p:cNvPicPr>\n      <a:pic'
            'Locks noChangeAspect="1"/>\n    </p:cNvPicPr>\n    <p:nvPr>\n  '
            '    <a:videoFile r:link="rId1"/>\n      <p:extLst>\n        <p:'
            'ext uri="{DAA4B4D4-6D71-4841-9C94-3DE7FCFB9230}">\n          <p'
            '14:media xmlns:p14="http://schemas.microsoft.com/office/powerpo'
            'int/2010/main" r:embed="rId2"/>\n        </p:ext>\n      </p:ex'
            "tLst>\n    </p:nvPr>\n  </p:nvPicPr>\n  <p:blipFill>\n    <a:bl"
            'ip r:embed="rId3"/>\n    <a:stretch>\n      <a:fillRect/>\n    '
            "</a:stretch>\n  </p:blipFill>\n  <p:spPr>\n    <a:xfrm>\n      "
            '<a:off x="1" y="2"/>\n      <a:ext cx="3" cy="4"/>\n    </a:xfr'
            'm>\n    <a:prstGeom prst="rect">\n      <a:avLst/>\n    </a:prs'
            "tGeom>\n  </p:spPr>\n</p:pic>\n"
        )
        return (
            shape_id,
            shape_name,
            video_rId,
            media_rId,
            poster_frame_rId,
            x,
            y,
            cx,
            cy,
            expected_xml,
        )



# encoding: utf-8

"""
Test suite for pptx.shapes.shape module
"""

from __future__ import absolute_import

import pytest

from pptx.action import ActionSetting
from pptx.dml.effect import ShadowFormat
from pptx.enum.shapes import PP_PLACEHOLDER
from pptx.oxml.shapes.shared import BaseShapeElement
from pptx.oxml.text import CT_TextBody
from pptx.shapes import Subshape
from pptx.shapes.autoshape import Shape
from pptx.shapes.base import BaseShape, _PlaceholderFormat
from pptx.shapes.graphfrm import GraphicFrame
from pptx.shapes.picture import Picture
from pptx.shapes.shapetree import BaseShapeFactory, SlideShapes

from ..oxml.unitdata.shape import (
    a_cNvPr,
    a_cxnSp,
    a_graphicFrame,
    a_grpSp,
    a_grpSpPr,
    a_p_xfrm,
    a_pic,
    an_ext,
    an_nvSpPr,
    an_off,
    an_sp,
    an_spPr,
    an_xfrm,
)
from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock, loose_mock, property_mock


class DescribeBaseShape(object):
    def it_provides_access_to_its_click_action(self, click_action_fixture):
        shape, ActionSetting_, cNvPr, click_action_ = click_action_fixture
        click_action = shape.click_action
        ActionSetting_.assert_called_once_with(cNvPr, shape)
        assert click_action is click_action_

    def it_knows_its_shape_id(self, id_fixture):
        shape, expected_value = id_fixture
        assert shape.shape_id == expected_value

    def it_knows_its_name(self, name_get_fixture):
        shape, name = name_get_fixture
        assert shape.name == name

    def it_can_change_its_name(self, name_set_fixture):
        shape, new_value, expected_xml = name_set_fixture
        shape.name = new_value
        assert shape._element.xml == expected_xml

    def it_has_a_position(self, position_get_fixture):
        shape, expected_left, expected_top = position_get_fixture
        assert shape.left == expected_left
        assert shape.top == expected_top

    def it_can_change_its_position(self, position_set_fixture):
        shape, left, top, expected_xml = position_set_fixture
        shape.left = left
        shape.top = top
        assert shape._element.xml == expected_xml

    def it_has_dimensions(self, dimensions_get_fixture):
        shape, expected_width, expected_height = dimensions_get_fixture
        assert shape.width == expected_width
        assert shape.height == expected_height

    def it_can_change_its_dimensions(self, dimensions_set_fixture):
        shape, width, height, expected_xml = dimensions_set_fixture
        shape.width = width
        shape.height = height
        assert shape._element.xml == expected_xml

    def it_knows_its_rotation_angle(self, rotation_get_fixture):
        shape, expected_value = rotation_get_fixture
        assert shape.rotation == expected_value

    def it_can_change_its_rotation_angle(self, rotation_set_fixture):
        shape, new_value, expected_xml = rotation_set_fixture
        shape.rotation = new_value
        assert shape._element.xml == expected_xml

    def it_provides_access_to_its_shadow(self, shadow_fixture):
        shape, ShadowFormat_, spPr, shadow_ = shadow_fixture

        shadow = shape.shadow

        ShadowFormat_.assert_called_once_with(spPr)
        assert shadow is shadow_

    def it_knows_the_part_it_belongs_to(self, part_fixture):
        shape, parent_ = part_fixture
        part = shape.part
        assert part is parent_.part

    def it_knows_it_doesnt_have_a_text_frame(self):
        shape = BaseShape(None, None)
        assert shape.has_text_frame is False

    def it_knows_whether_it_is_a_placeholder(self, is_placeholder_fixture):
        shape, is_placeholder = is_placeholder_fixture
        assert shape.is_placeholder is is_placeholder

    def it_provides_access_to_its_placeholder_format(self, phfmt_fixture):
        shape, _PlaceholderFormat_, placeholder_format_, ph = phfmt_fixture
        placeholder_format = shape.placeholder_format
        _PlaceholderFormat_.assert_called_once_with(ph)
        assert placeholder_format is placeholder_format_

    def it_raises_when_shape_is_not_a_placeholder(self, phfmt_raise_fixture):
        shape = phfmt_raise_fixture
        with pytest.raises(ValueError):
            shape.placeholder_format

    def it_knows_it_doesnt_contain_a_chart(self):
        shape = BaseShape(None, None)
        assert shape.has_chart is False

    def it_knows_it_doesnt_contain_a_table(self):
        shape = BaseShape(None, None)
        assert shape.has_table is False

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            "p:sp/p:nvSpPr/p:cNvPr",
            "p:grpSp/p:nvGrpSpPr/p:cNvPr",
            "p:graphicFrame/p:nvGraphicFramePr/p:cNvPr",
            "p:cxnSp/p:nvCxnSpPr/p:cNvPr",
            "p:pic/p:nvPicPr/p:cNvPr",
        ]
    )
    def click_action_fixture(self, request, ActionSetting_, action_setting_):
        sp_cxml = request.param
        sp = element(sp_cxml)
        cNvPr = sp.xpath("//p:cNvPr")[0]
        shape = BaseShape(sp, None)
        return shape, ActionSetting_, cNvPr, action_setting_

    @pytest.fixture(
        params=[
            ("sp", False),
            ("sp_with_ext", True),
            ("pic", False),
            ("pic_with_ext", True),
            ("graphicFrame", False),
            ("graphicFrame_with_ext", True),
            ("grpSp", False),
            ("grpSp_with_ext", True),
            ("cxnSp", False),
            ("cxnSp_with_ext", True),
        ]
    )
    def dimensions_get_fixture(self, request, width, height):
        shape_elm_fixt_name, expect_values = request.param
        shape_elm = request.getfixturevalue(shape_elm_fixt_name)
        shape = BaseShape(shape_elm, None)
        if not expect_values:
            width = height = None
        return shape, width, height

    @pytest.fixture(
        params=[
            ("sp", "sp_with_ext"),
            ("pic", "pic_with_ext"),
            ("graphicFrame", "graphicFrame_with_ext"),
            ("grpSp", "grpSp_with_ext"),
            ("cxnSp", "cxnSp_with_ext"),
        ]
    )
    def dimensions_set_fixture(self, request, width, height):
        start_elm_fixt_name, expected_elm_fixt_name = request.param
        start_elm = request.getfixturevalue(start_elm_fixt_name)
        shape = BaseShape(start_elm, None)
        expected_xml = request.getfixturevalue(expected_elm_fixt_name).xml
        return shape, width, height, expected_xml

    @pytest.fixture(
        params=[
            ("p:sp/p:nvSpPr/p:cNvPr{id=1}", 1),
            ("p:cxnSp/p:nvCxnSpPr/p:cNvPr{id=2}", 2),
            ("p:graphicFrame/p:nvGraphicFramePr/p:cNvPr{id=3}", 3),
            ("p:grpSp/p:nvGrpSpPr/p:cNvPr{id=4}", 4),
            ("p:pic/p:nvPicPr/p:cNvPr{id=5}", 5),
        ]
    )
    def id_fixture(self, request):
        xSp_cxml, expected_value = request.param
        shape = BaseShape(element(xSp_cxml), None)
        return shape, expected_value

    @pytest.fixture(params=[True, False])
    def is_placeholder_fixture(self, request, shape_elm_, txBody_):
        is_placeholder = request.param
        shape_elm_.has_ph_elm = is_placeholder
        shape = BaseShape(shape_elm_, None)
        return shape, is_placeholder

    @pytest.fixture
    def name_get_fixture(self, shape_name):
        shape_elm = (
            an_sp()
            .with_nsdecls()
            .with_child(an_nvSpPr().with_child(a_cNvPr().with_name(shape_name)))
        ).element
        shape = BaseShape(shape_elm, None)
        return shape, shape_name

    @pytest.fixture(
        params=[
            (
                "p:sp/p:nvSpPr/p:cNvPr{id=1,name=foo}",
                Shape,
                "Shape1",
                "p:sp/p:nvSpPr/p:cNvPr{id=1,name=Shape1}",
            ),
            (
                "p:grpSp/p:nvGrpSpPr/p:cNvPr{id=2,name=bar}",
                BaseShape,
                "Shape2",
                "p:grpSp/p:nvGrpSpPr/p:cNvPr{id=2,name=Shape2}",
            ),
            (
                "p:graphicFrame/p:nvGraphicFramePr/p:cNvPr{id=3,name=baz}",
                GraphicFrame,
                "Shape3",
                "p:graphicFrame/p:nvGraphicFramePr/p:cNvPr{id=3,name=Shape3}",
            ),
            (
                "p:cxnSp/p:nvCxnSpPr/p:cNvPr{id=4,name=boo}",
                BaseShape,
                "Shape4",
                "p:cxnSp/p:nvCxnSpPr/p:cNvPr{id=4,name=Shape4}",
            ),
            (
                "p:pic/p:nvPicPr/p:cNvPr{id=5,name=far}",
                Picture,
                "Shape5",
                "p:pic/p:nvPicPr/p:cNvPr{id=5,name=Shape5}",
            ),
        ]
    )
    def name_set_fixture(self, request):
        xSp_cxml, ShapeCls, new_value, expected_xSp_cxml = request.param
        shape = ShapeCls(element(xSp_cxml), None)
        expected_xml = xml(expected_xSp_cxml)
        return shape, new_value, expected_xml

    @pytest.fixture
    def part_fixture(self, shapes_):
        parent_ = shapes_
        shape = BaseShape(None, parent_)
        return shape, parent_

    @pytest.fixture
    def phfmt_fixture(self, _PlaceholderFormat_, placeholder_format_):
        sp = element("p:sp/p:nvSpPr/p:nvPr/p:ph")
        ph = sp.xpath("//p:ph")[0]
        shape = BaseShape(sp, None)
        return shape, _PlaceholderFormat_, placeholder_format_, ph

    @pytest.fixture
    def phfmt_raise_fixture(self):
        return BaseShape(element("p:sp/p:nvSpPr/p:nvPr"), None)

    @pytest.fixture(
        params=[
            ("sp", False),
            ("sp_with_off", True),
            ("pic", False),
            ("pic_with_off", True),
            ("graphicFrame", False),
            ("graphicFrame_with_off", True),
            ("grpSp", False),
            ("grpSp_with_off", True),
            ("cxnSp", False),
            ("cxnSp_with_off", True),
        ]
    )
    def position_get_fixture(self, request, left, top):
        shape_elm_fixt_name, expect_values = request.param
        shape_elm = request.getfixturevalue(shape_elm_fixt_name)
        shape = BaseShape(shape_elm, None)
        if not expect_values:
            left = top = None
        return shape, left, top

    @pytest.fixture(
        params=[
            ("sp", "sp_with_off"),
            ("pic", "pic_with_off"),
            ("graphicFrame", "graphicFrame_with_off"),
            ("grpSp", "grpSp_with_off"),
            ("cxnSp", "cxnSp_with_off"),
        ]
    )
    def position_set_fixture(self, request, left, top):
        start_elm_fixt_name, expected_elm_fixt_name = request.param
        start_elm = request.getfixturevalue(start_elm_fixt_name)
        shape = BaseShape(start_elm, None)
        expected_xml = request.getfixturevalue(expected_elm_fixt_name).xml
        return shape, left, top, expected_xml

    @pytest.fixture(
        params=[
            ("p:sp/p:spPr", 0.0),
            ("p:sp/p:spPr/a:xfrm{rot=60000}", 1.0),
            ("p:sp/p:spPr/a:xfrm{rot=2545200}", 42.42),
            ("p:sp/p:spPr/a:xfrm{rot=-60000}", 359.0),
            ("p:grpSp/p:grpSpPr/a:xfrm{rot=2545200}", 42.42),
        ]
    )
    def rotation_get_fixture(self, request):
        xSp_cxml, expected_value = request.param
        shape = BaseShapeFactory(element(xSp_cxml), None)
        return shape, expected_value

    @pytest.fixture(
        params=[
            ("p:sp/p:spPr/a:xfrm", 1.0, "p:sp/p:spPr/a:xfrm{rot=60000}"),
            ("p:sp/p:spPr/a:xfrm{rot=60000}", 0.0, "p:sp/p:spPr/a:xfrm"),
            (
                "p:sp/p:spPr/a:xfrm{rot=60000}",
                -420.0,
                "p:sp/p:spPr/a:xfrm{rot=18000000}",
            ),
            ("p:grpSp/p:grpSpPr/a:xfrm", 1.0, "p:grpSp/p:grpSpPr/a:xfrm{rot=60000}"),
        ]
    )
    def rotation_set_fixture(self, request):
        xSp_cxml, new_value, expected_xSp_cxml = request.param
        shape = BaseShapeFactory(element(xSp_cxml), None)
        expected_xml = xml(expected_xSp_cxml)
        return shape, new_value, expected_xml

    @pytest.fixture(
        params=[
            "p:sp/p:spPr",
            "p:cxnSp/p:spPr",
            "p:pic/p:spPr",
            # ---group and graphic frame shapes override this property---
        ]
    )
    def shadow_fixture(self, request, ShadowFormat_, shadow_):
        sp_cxml = request.param
        sp = element(sp_cxml)
        spPr = sp.xpath("//p:spPr")[0]
        ShadowFormat_.return_value = shadow_

        shape = BaseShape(sp, None)
        return shape, ShadowFormat_, spPr, shadow_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ActionSetting_(self, request, action_setting_):
        return class_mock(
            request, "pptx.shapes.base.ActionSetting", return_value=action_setting_
        )

    @pytest.fixture
    def action_setting_(self, request):
        return instance_mock(request, ActionSetting)

    @pytest.fixture
    def cxnSp(self):
        return a_cxnSp().with_nsdecls().with_child(an_spPr()).element

    @pytest.fixture
    def cxnSp_with_ext(self, width, height):
        return (
            a_cxnSp()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_ext().with_cx(width).with_cy(height))
                )
            )
        ).element

    @pytest.fixture
    def cxnSp_with_off(self, left, top):
        return (
            a_cxnSp()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_off().with_x(left).with_y(top))
                )
            )
        ).element

    @pytest.fixture
    def graphicFrame(self):
        # Note that <p:xfrm> element is required on graphicFrame
        return a_graphicFrame().with_nsdecls().with_child(a_p_xfrm()).element

    @pytest.fixture
    def graphicFrame_with_ext(self, width, height):
        return (
            a_graphicFrame()
            .with_nsdecls()
            .with_child(a_p_xfrm().with_child(an_ext().with_cx(width).with_cy(height)))
        ).element

    @pytest.fixture
    def graphicFrame_with_off(self, left, top):
        return (
            a_graphicFrame()
            .with_nsdecls()
            .with_child(a_p_xfrm().with_child(an_off().with_x(left).with_y(top)))
        ).element

    @pytest.fixture
    def grpSp(self):
        return (a_grpSp().with_nsdecls("p", "a").with_child(a_grpSpPr())).element

    @pytest.fixture
    def grpSp_with_ext(self, width, height):
        return (
            a_grpSp()
            .with_nsdecls("p", "a")
            .with_child(
                a_grpSpPr().with_child(
                    an_xfrm().with_child(an_ext().with_cx(width).with_cy(height))
                )
            )
        ).element

    @pytest.fixture
    def grpSp_with_off(self, left, top):
        return (
            a_grpSp()
            .with_nsdecls("p", "a")
            .with_child(
                a_grpSpPr().with_child(
                    an_xfrm().with_child(an_off().with_x(left).with_y(top))
                )
            )
        ).element

    @pytest.fixture
    def height(self):
        return 654

    @pytest.fixture
    def left(self):
        return 123

    @pytest.fixture
    def pic(self):
        return a_pic().with_nsdecls().with_child(an_spPr()).element

    @pytest.fixture
    def pic_with_off(self, left, top):
        return (
            a_pic()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_off().with_x(left).with_y(top))
                )
            )
        ).element

    @pytest.fixture
    def pic_with_ext(self, width, height):
        return (
            a_pic()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_ext().with_cx(width).with_cy(height))
                )
            )
        ).element

    @pytest.fixture
    def _PlaceholderFormat_(self, request, placeholder_format_):
        return class_mock(
            request,
            "pptx.shapes.base._PlaceholderFormat",
            return_value=placeholder_format_,
        )

    @pytest.fixture
    def placeholder_format_(self, request):
        return instance_mock(request, _PlaceholderFormat)

    @pytest.fixture
    def shadow_(self, request):
        return instance_mock(request, ShadowFormat)

    @pytest.fixture
    def ShadowFormat_(self, request):
        return class_mock(request, "pptx.shapes.base.ShadowFormat")

    @pytest.fixture
    def shape_elm_(self, request, shape_id, shape_name, txBody_):
        return instance_mock(
            request,
            BaseShapeElement,
            shape_id=shape_id,
            shape_name=shape_name,
            txBody=txBody_,
        )

    @pytest.fixture
    def shape_id(self):
        return 42

    @pytest.fixture
    def shape_name(self):
        return "Foobar 41"

    @pytest.fixture
    def shape_text_frame_(self, request):
        return property_mock(request, BaseShape, "text_frame")

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, SlideShapes)

    @pytest.fixture
    def sp(self):
        return an_sp().with_nsdecls().with_child(an_spPr()).element

    @pytest.fixture
    def sp_with_ext(self, width, height):
        return (
            an_sp()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_ext().with_cx(width).with_cy(height))
                )
            )
        ).element

    @pytest.fixture
    def sp_with_off(self, left, top):
        return (
            an_sp()
            .with_nsdecls()
            .with_child(
                an_spPr().with_child(
                    an_xfrm().with_child(an_off().with_x(left).with_y(top))
                )
            )
        ).element

    @pytest.fixture
    def top(self):
        return 456

    @pytest.fixture
    def txBody_(self, request):
        return instance_mock(request, CT_TextBody)

    @pytest.fixture
    def width(self):
        return 321


class DescribeSubshape(object):
    def it_knows_the_part_it_belongs_to(self, subshape_with_parent_):
        subshape, parent_ = subshape_with_parent_
        part = subshape.part
        assert part is parent_.part

    # fixtures ---------------------------------------------

    @pytest.fixture
    def subshape_with_parent_(self, request):
        parent_ = loose_mock(request, name="parent_")
        subshape = Subshape(parent_)
        return subshape, parent_


class Describe_PlaceholderFormat(object):
    def it_knows_its_idx(self, idx_get_fixture):
        placeholder_format, expected_value = idx_get_fixture
        assert placeholder_format.idx == expected_value

    def it_knows_its_type(self, type_get_fixture):
        placeholder_format, expected_value = type_get_fixture
        assert placeholder_format.type == expected_value

    # fixtures ---------------------------------------------

    @pytest.fixture(params=[("p:ph", 0), ("p:ph{idx=42}", 42)])
    def idx_get_fixture(self, request):
        ph_cxml, expected_value = request.param
        placeholder_format = _PlaceholderFormat(element(ph_cxml))
        return placeholder_format, expected_value

    @pytest.fixture(
        params=[
            ("p:ph", PP_PLACEHOLDER.OBJECT),
            ("p:ph{type=pic}", PP_PLACEHOLDER.PICTURE),
        ]
    )
    def type_get_fixture(self, request):
        ph_cxml, expected_value = request.param
        placeholder_format = _PlaceholderFormat(element(ph_cxml))
        return placeholder_format, expected_value



# encoding: utf-8

"""
Placeholder-related objects, specific to shapes having a `p:ph` element.
A placeholder has distinct behaviors depending on whether it appears on
a slide, layout, or master. Hence there is a non-trivial class inheritance
structure.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from .autoshape import Shape
from ..enum.shapes import MSO_SHAPE_TYPE, PP_PLACEHOLDER
from .graphfrm import GraphicFrame
from ..oxml.shapes.graphfrm import CT_GraphicalObjectFrame
from ..oxml.shapes.picture import CT_Picture
from .picture import Picture
from ..util import Emu


class _InheritsDimensions(object):
    """
    Mixin class that provides inherited dimension behavior. Specifically,
    left, top, width, and height report the value from the layout placeholder
    where they would have otherwise reported |None|. This behavior is
    distinctive to placeholders. :meth:`_base_placeholder` must be overridden
    by all subclasses to provide lookup of the appropriate base placeholder
    to inherit from.
    """

    @property
    def height(self):
        """
        The effective height of this placeholder shape; its directly-applied
        height if it has one, otherwise the height of its parent layout
        placeholder.
        """
        return self._effective_value("height")

    @height.setter
    def height(self, value):
        self._element.cy = value

    @property
    def left(self):
        """
        The effective left of this placeholder shape; its directly-applied
        left if it has one, otherwise the left of its parent layout
        placeholder.
        """
        return self._effective_value("left")

    @left.setter
    def left(self, value):
        self._element.x = value

    @property
    def shape_type(self):
        """
        Member of :ref:`MsoShapeType` specifying the type of this shape.
        Unconditionally ``MSO_SHAPE_TYPE.PLACEHOLDER`` in this case.
        Read-only.
        """
        return MSO_SHAPE_TYPE.PLACEHOLDER

    @property
    def top(self):
        """
        The effective top of this placeholder shape; its directly-applied
        top if it has one, otherwise the top of its parent layout
        placeholder.
        """
        return self._effective_value("top")

    @top.setter
    def top(self, value):
        self._element.y = value

    @property
    def width(self):
        """
        The effective width of this placeholder shape; its directly-applied
        width if it has one, otherwise the width of its parent layout
        placeholder.
        """
        return self._effective_value("width")

    @width.setter
    def width(self, value):
        self._element.cx = value

    @property
    def _base_placeholder(self):
        """
        Return the layout or master placeholder shape this placeholder
        inherits from. Not to be confused with an instance of
        |BasePlaceholder| (necessarily).
        """
        raise NotImplementedError("Must be implemented by all subclasses.")

    def _effective_value(self, attr_name):
        """
        The effective value of *attr_name* on this placeholder shape; its
        directly-applied value if it has one, otherwise the value on the
        layout placeholder it inherits from.
        """
        directly_applied_value = getattr(super(_InheritsDimensions, self), attr_name)
        if directly_applied_value is not None:
            return directly_applied_value
        return self._inherited_value(attr_name)

    def _inherited_value(self, attr_name):
        """
        Return the attribute value, e.g. 'width' of the base placeholder this
        placeholder inherits from.
        """
        base_placeholder = self._base_placeholder
        if base_placeholder is None:
            return None
        inherited_value = getattr(base_placeholder, attr_name)
        return inherited_value


class _BaseSlidePlaceholder(_InheritsDimensions, Shape):
    """
    Base class for placeholders on slides. Provides common behaviors such as
    inherited dimensions.
    """

    @property
    def is_placeholder(self):
        """
        Boolean indicating whether this shape is a placeholder.
        Unconditionally |True| in this case.
        """
        return True

    @property
    def shape_type(self):
        """
        Member of :ref:`MsoShapeType` specifying the type of this shape.
        Unconditionally ``MSO_SHAPE_TYPE.PLACEHOLDER`` in this case.
        Read-only.
        """
        return MSO_SHAPE_TYPE.PLACEHOLDER

    @property
    def _base_placeholder(self):
        """
        Return the layout placeholder this slide placeholder inherits from.
        Not to be confused with an instance of |BasePlaceholder|
        (necessarily).
        """
        layout, idx = self.part.slide_layout, self._element.ph_idx
        return layout.placeholders.get(idx=idx)

    def _replace_placeholder_with(self, element):
        """
        Substitute *element* for this placeholder element in the shapetree.
        This placeholder's `._element` attribute is set to |None| and its
        original element is free for garbage collection. Any attribute access
        (including a method call) on this placeholder after this call raises
        |AttributeError|.
        """
        element._nvXxPr.nvPr._insert_ph(self._element.ph)
        self._element.addprevious(element)
        self._element.getparent().remove(self._element)
        self._element = None


class BasePlaceholder(Shape):
    """
    NOTE: This class is deprecated and will be removed from a future release
    along with the properties *idx*, *orient*, *ph_type*, and *sz*. The *idx*
    property will be available via the .placeholder_format property. The
    others will be accessed directly from the oxml layer as they are only
    used for internal purposes.

    Base class for placeholder subclasses that differentiate the varying
    behaviors of placeholders on a master, layout, and slide.
    """

    @property
    def idx(self):
        """
        Integer placeholder 'idx' attribute, e.g. 0
        """
        return self._sp.ph_idx

    @property
    def orient(self):
        """
        Placeholder orientation, e.g. ST_Direction.HORZ
        """
        return self._sp.ph_orient

    @property
    def ph_type(self):
        """
        Placeholder type, e.g. PP_PLACEHOLDER.CENTER_TITLE
        """
        return self._sp.ph_type

    @property
    def sz(self):
        """
        Placeholder 'sz' attribute, e.g. ST_PlaceholderSize.FULL
        """
        return self._sp.ph_sz


class LayoutPlaceholder(_InheritsDimensions, Shape):
    """
    Placeholder shape on a slide layout, providing differentiated behavior
    for slide layout placeholders, in particular, inheriting shape properties
    from the master placeholder having the same type, when a matching one
    exists.
    """

    @property
    def _base_placeholder(self):
        """
        Return the master placeholder this layout placeholder inherits from.
        """
        base_ph_type = {
            PP_PLACEHOLDER.BODY: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.CHART: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.BITMAP: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.CENTER_TITLE: PP_PLACEHOLDER.TITLE,
            PP_PLACEHOLDER.ORG_CHART: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.DATE: PP_PLACEHOLDER.DATE,
            PP_PLACEHOLDER.FOOTER: PP_PLACEHOLDER.FOOTER,
            PP_PLACEHOLDER.MEDIA_CLIP: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.OBJECT: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.PICTURE: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.SLIDE_NUMBER: PP_PLACEHOLDER.SLIDE_NUMBER,
            PP_PLACEHOLDER.SUBTITLE: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.TABLE: PP_PLACEHOLDER.BODY,
            PP_PLACEHOLDER.TITLE: PP_PLACEHOLDER.TITLE,
        }[self._element.ph_type]
        slide_master = self.part.slide_master
        return slide_master.placeholders.get(base_ph_type, None)


class MasterPlaceholder(BasePlaceholder):
    """
    Placeholder shape on a slide master.
    """


class NotesSlidePlaceholder(_InheritsDimensions, Shape):
    """
    Placeholder shape on a notes slide. Inherits shape properties from the
    placeholder on the notes master that has the same type (e.g. 'body').
    """

    @property
    def _base_placeholder(self):
        """
        Return the notes master placeholder this notes slide placeholder
        inherits from, or |None| if no placeholder of the matching type is
        present.
        """
        notes_master = self.part.notes_master
        ph_type = self.element.ph_type
        return notes_master.placeholders.get(ph_type=ph_type)


class SlidePlaceholder(_BaseSlidePlaceholder):
    """
    Placeholder shape on a slide. Inherits shape properties from its
    corresponding slide layout placeholder.
    """


class ChartPlaceholder(_BaseSlidePlaceholder):
    """
    Placeholder shape that can only accept a chart.
    """

    def insert_chart(self, chart_type, chart_data):
        """
        Return a |PlaceholderGraphicFrame| object containing a new chart of
        *chart_type* depicting *chart_data* and having the same position and
        size as this placeholder. *chart_type* is one of the
        :ref:`XlChartType` enumeration values. *chart_data* is a |ChartData|
        object populated with the categories and series values for the chart.
        Note that the new |Chart| object is not returned directly. The chart
        object may be accessed using the
        :attr:`~.PlaceholderGraphicFrame.chart` property of the returned
        |PlaceholderGraphicFrame| object.
        """
        rId = self.part.add_chart_part(chart_type, chart_data)
        graphicFrame = self._new_chart_graphicFrame(
            rId, self.left, self.top, self.width, self.height
        )
        self._replace_placeholder_with(graphicFrame)
        return PlaceholderGraphicFrame(graphicFrame, self._parent)

    def _new_chart_graphicFrame(self, rId, x, y, cx, cy):
        """
        Return a newly created `p:graphicFrame` element having the specified
        position and size and containing the chart identified by *rId*.
        """
        id_, name = self.shape_id, self.name
        return CT_GraphicalObjectFrame.new_chart_graphicFrame(
            id_, name, rId, x, y, cx, cy
        )


class PicturePlaceholder(_BaseSlidePlaceholder):
    """
    Placeholder shape that can only accept a picture.
    """

    def insert_picture(self, image_file):
        """
        Return a |PlaceholderPicture| object depicting the image in
        *image_file*, which may be either a path (string) or a file-like
        object. The image is cropped to fill the entire space of the
        placeholder. A |PlaceholderPicture| object has all the properties and
        methods of a |Picture| shape except that the value of its
        :attr:`~._BaseSlidePlaceholder.shape_type` property is
        `MSO_SHAPE_TYPE.PLACEHOLDER` instead of `MSO_SHAPE_TYPE.PICTURE`.
        """
        pic = self._new_placeholder_pic(image_file)
        self._replace_placeholder_with(pic)
        return PlaceholderPicture(pic, self._parent)

    def _new_placeholder_pic(self, image_file):
        """
        Return a new `p:pic` element depicting the image in *image_file*,
        suitable for use as a placeholder. In particular this means not
        having an `a:xfrm` element, allowing its extents to be inherited from
        its layout placeholder.
        """
        rId, desc, image_size = self._get_or_add_image(image_file)
        shape_id, name = self.shape_id, self.name
        pic = CT_Picture.new_ph_pic(shape_id, name, desc, rId)
        pic.crop_to_fit(image_size, (self.width, self.height))
        return pic

    def _get_or_add_image(self, image_file):
        """
        Return an (rId, description, image_size) 3-tuple identifying the
        related image part containing *image_file* and describing the image.
        """
        image_part, rId = self.part.get_or_add_image_part(image_file)
        desc, image_size = image_part.desc, image_part._px_size
        return rId, desc, image_size


class PlaceholderGraphicFrame(GraphicFrame):
    """
    Placeholder shape populated with a table, chart, or smart art.
    """

    @property
    def is_placeholder(self):
        """
        Boolean indicating whether this shape is a placeholder.
        Unconditionally |True| in this case.
        """
        return True


class PlaceholderPicture(_InheritsDimensions, Picture):
    """
    Placeholder shape populated with a picture.
    """

    @property
    def _base_placeholder(self):
        """
        Return the layout placeholder this picture placeholder inherits from.
        """
        layout, idx = self.part.slide_layout, self._element.ph_idx
        return layout.placeholders.get(idx=idx)


class TablePlaceholder(_BaseSlidePlaceholder):
    """
    Placeholder shape that can only accept a picture.
    """

    def insert_table(self, rows, cols):
        """
        Return a |PlaceholderGraphicFrame| object containing a table of
        *rows* rows and *cols* columns. The position and width of the table
        are those of the placeholder and its height is proportional to the
        number of rows. A |PlaceholderGraphicFrame| object has all the
        properties and methods of a |GraphicFrame| shape except that the
        value of its :attr:`~._BaseSlidePlaceholder.shape_type` property is
        unconditionally `MSO_SHAPE_TYPE.PLACEHOLDER`. Note that the return
        value is not the new table but rather *contains* the new table. The
        table can be accessed using the
        :attr:`~.PlaceholderGraphicFrame.table` property of the returned
        |PlaceholderGraphicFrame| object.
        """
        graphicFrame = self._new_placeholder_table(rows, cols)
        self._replace_placeholder_with(graphicFrame)
        return PlaceholderGraphicFrame(graphicFrame, self._parent)

    def _new_placeholder_table(self, rows, cols):
        """
        Return a newly added `p:graphicFrame` element containing an empty
        table with *rows* rows and *cols* columns, positioned at the location
        of this placeholder and having its same width. The table's height is
        determined by the number of rows.
        """
        shape_id, name, height = self.shape_id, self.name, Emu(rows * 370840)
        return CT_GraphicalObjectFrame.new_table_graphicFrame(
            shape_id, name, rows, cols, self.left, self.top, self.width, height
        )



# encoding: utf-8

"""Unit test suite for the pptx.chart.datalabel module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.chart.datalabel import DataLabel, DataLabels
from pptx.enum.chart import XL_LABEL_POSITION
from pptx.text.text import Font

from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock, method_mock


class DescribeDataLabel(object):
    def it_has_a_font(self, font_fixture):
        data_label, expected_xml = font_fixture
        font = data_label.font
        assert data_label._ser.xml == expected_xml
        assert isinstance(font, Font)

    def it_knows_its_position(self, position_get_fixture):
        data_label, expected_value = position_get_fixture
        position = data_label.position
        assert position == expected_value

    def it_can_change_its_position(self, position_set_fixture):
        data_label, value, expected_xml = position_set_fixture
        data_label.position = value
        assert data_label._element.xml == expected_xml

    def it_knows_whether_it_has_a_text_frame(self, has_tf_get_fixture):
        data_label, expected_value = has_tf_get_fixture
        value = data_label.has_text_frame
        assert value is expected_value

    def it_can_change_whether_it_has_a_text_frame(self, has_tf_set_fixture):
        data_label, value, expected_xml = has_tf_set_fixture
        data_label.has_text_frame = value
        assert data_label._element.xml == expected_xml

    def it_provides_access_to_its_text_frame(self, text_frame_fixture):
        data_label, TextFrame_, rich_, text_frame_ = text_frame_fixture
        text_frame = data_label.text_frame
        TextFrame_.assert_called_once_with(rich_, data_label)
        assert text_frame is text_frame_

    def it_gets_or_adds_rich_element_to_help(self, rich_fixture):
        data_label, expected_xml = rich_fixture
        rich = data_label._get_or_add_rich()
        assert data_label._ser.xml == expected_xml
        assert rich is data_label._ser.xpath("c:dLbls/c:dLbl/c:tx/c:rich")[0]

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            (
                "c:ser{a:b=c}",
                "c:ser{a:b=c}/c:dLbls/(c:dLbl/(c:idx{val=9},c:spPr,c:txPr/(a:bodyPr"
                ",a:lstStyle,a:p/a:pPr/a:defRPr),c:showLegendKey{val=0},c:showVal{v"
                "al=1},c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val="
                "0},c:showBubbleSize{val=0}),c:showLegendKey{val=0},c:showVal{val=0"
                "},c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0},c"
                ":showBubbleSize{val=0},c:showLeaderLines{val=1})",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=9},c:txPr/(a:bodyPr,a:p))",
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=9},c:txPr/(a:bodyPr,a:p/a:p"
                "Pr/a:defRPr))",
            ),
        ]
    )
    def font_fixture(self, request):
        ser_cxml, expected_cxml = request.param
        data_label = DataLabel(element(ser_cxml), 9)
        expected_xml = xml(expected_cxml)
        return data_label, expected_xml

    @pytest.fixture(
        params=[
            ("c:ser", False),
            ("c:ser/c:dLbls", False),
            ("c:ser/c:dLbls/c:dLbl/c:idx{val=42}", False),
            ("c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:strRef)", False),
            ("c:ser/c:dLbls/c:dLbl/(c:idx{val=24},c:tx/c:rich)", False),
            ("c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich)", True),
        ]
    )
    def has_tf_get_fixture(self, request):
        ser_cxml, expected_value = request.param
        data_label = DataLabel(element(ser_cxml), 42)
        return data_label, expected_value

    @pytest.fixture(
        params=[
            ("c:ser", False, "c:ser"),
            ("c:ser/c:dLbls", False, "c:ser/c:dLbls"),
            (
                "c:ser/c:dLbls/c:dLbl/c:idx{val=42}",
                False,
                "c:ser/c:dLbls/c:dLbl/c:idx{val=42}",
            ),
            (
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:strRef)",
                False,
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:strRef)",
            ),
            (
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=24},c:tx/c:rich)",
                False,
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=24},c:tx/c:rich)",
            ),
            (
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich)",
                False,
                "c:ser/c:dLbls/c:dLbl/c:idx{val=42}",
            ),
            (
                "c:ser{a:b=c}",
                True,
                "c:ser{a:b=c}/c:dLbls/(c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,"
                "a:lstStyle,a:p/a:pPr/a:defRPr),c:showLegendKey{val=0},c:showVal{va"
                "l=1},c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0"
                "},c:showBubbleSize{val=0}),c:showLegendKey{val=0},c:showVal{val=0}"
                ",c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0},c:"
                "showBubbleSize{val=0},c:showLeaderLines{val=1})",
            ),
            (
                "c:ser{a:b=c}/c:dLbls",
                True,
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,a"
                ":lstStyle,a:p/a:pPr/a:defRPr),c:showLegendKey{val=0},c:showVal{val"
                "=1},c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0}"
                ",c:showBubbleSize{val=0})",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/c:idx{val=42}",
                True,
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,a"
                ":lstStyle,a:p/a:pPr/a:defRPr))",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:strRef)",
                True,
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,a"
                ":lstStyle,a:p/a:pPr/a:defRPr))",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=24},c:tx/c:rich)",
                True,
                "c:ser{a:b=c}/c:dLbls/(c:dLbl/(c:idx{val=24},c:tx/c:rich),c:dLbl/(c"
                ":idx{val=42},c:tx/c:rich/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr),"
                "c:showLegendKey{val=0},c:showVal{val=1},c:showCatName{val=0},c:sho"
                "wSerName{val=0},c:showPercent{val=0},c:showBubbleSize{val=0}))",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/c:idx{val=42}",
                True,
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,a"
                ":lstStyle,a:p/a:pPr/a:defRPr))",
            ),
        ]
    )
    def has_tf_set_fixture(self, request):
        ser_cxml, value, expected_cxml = request.param
        data_label = DataLabel(element(ser_cxml), 42)
        expected_xml = xml(expected_cxml)
        return data_label, value, expected_xml

    @pytest.fixture(
        params=[
            ("c:ser", None),
            ("c:ser/c:dLbls/c:dLbl/c:idx{val=42}", None),
            ("c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:dLblPos{val=b})", "BELOW"),
        ]
    )
    def position_get_fixture(self, request):
        ser_cxml, value = request.param
        data_label = DataLabel(element(ser_cxml), 42)
        expected_value = None if value is None else getattr(XL_LABEL_POSITION, value)
        return data_label, expected_value

    @pytest.fixture(
        params=[
            (
                "c:ser{a:b=c}",
                "CENTER",
                "c:ser{a:b=c}/c:dLbls/(c:dLbl/(c:idx{val=42},c:spPr,c:txPr/(a:bodyP"
                "r,a:lstStyle,a:p/a:pPr/a:defRPr),c:dLblPos{val=ctr},c:showLegendKe"
                "y{val=0},c:showVal{val=1},c:showCatName{val=0},c:showSerName{val=0"
                "},c:showPercent{val=0},c:showBubbleSize{val=0}),c:showLegendKey{va"
                "l=0},c:showVal{val=0},c:showCatName{val=0},c:showSerName{val=0},c:"
                "showPercent{val=0},c:showBubbleSize{val=0},c:showLeaderLines{val=1"
                "})",
            ),
            (
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:dLblPos{val=ctr})",
                "BELOW",
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:dLblPos{val=b})",
            ),
            (
                "c:ser/c:dLbls/c:dLbl/(c:idx{val=42},c:dLblPos{val=b})",
                None,
                "c:ser/c:dLbls/c:dLbl/c:idx{val=42}",
            ),
            ("c:ser", None, "c:ser"),
        ]
    )
    def position_set_fixture(self, request):
        ser_cxml, value, expected_cxml = request.param
        data_label = DataLabel(element(ser_cxml), 42)
        new_value = None if value is None else getattr(XL_LABEL_POSITION, value)
        expected_xml = xml(expected_cxml)
        return data_label, new_value, expected_xml

    @pytest.fixture(
        params=[
            (
                "c:ser{a:b=c}",
                "c:ser{a:b=c}/c:dLbls/(c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,"
                "a:lstStyle,a:p/a:pPr/a:defRPr),c:showLegendKey{val=0},c:showVal{va"
                "l=1},c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0"
                "},c:showBubbleSize{val=0}),c:showLegendKey{val=0},c:showVal{val=0}"
                ",c:showCatName{val=0},c:showSerName{val=0},c:showPercent{val=0},c:"
                "showBubbleSize{val=0},c:showLeaderLines{val=1})",
            ),
            (
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:strRef)",
                "c:ser{a:b=c}/c:dLbls/c:dLbl/(c:idx{val=42},c:tx/c:rich/(a:bodyPr,a"
                ":lstStyle,a:p/a:pPr/a:defRPr))",
            ),
        ]
    )
    def rich_fixture(self, request):
        ser_cxml, expected_cxml = request.param
        data_label = DataLabel(element(ser_cxml), 42)
        expected_xml = xml(expected_cxml)
        return data_label, expected_xml

    @pytest.fixture
    def text_frame_fixture(self, request, _get_or_add_rich_, TextFrame_):
        data_label = DataLabel(None, None)
        rich_ = _get_or_add_rich_.return_value
        text_frame_ = TextFrame_.return_value
        return data_label, TextFrame_, rich_, text_frame_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _get_or_add_rich_(self, request):
        return method_mock(request, DataLabel, "_get_or_add_rich")

    @pytest.fixture
    def TextFrame_(self, request):
        return class_mock(request, "pptx.chart.datalabel.TextFrame")


class DescribeDataLabels(object):
    def it_provides_access_to_its_font(self, font_fixture):
        data_labels, Font_, defRPr, font_ = font_fixture
        font = data_labels.font
        Font_.assert_called_once_with(defRPr)
        assert font is font_

    def it_adds_a_txPr_to_help_font(self, txPr_fixture):
        data_labels, expected_xml = txPr_fixture
        data_labels.font
        assert data_labels._element.xml == expected_xml

    def it_knows_its_number_format(self, number_format_get_fixture):
        data_labels, expected_value = number_format_get_fixture
        assert data_labels.number_format == expected_value

    def it_can_change_its_number_format(self, number_format_set_fixture):
        data_labels, new_value, expected_xml = number_format_set_fixture
        data_labels.number_format = new_value
        assert data_labels._element.xml == expected_xml

    def it_knows_whether_its_number_format_is_linked(
        self, number_format_is_linked_get_fixture
    ):
        data_labels, expected_value = number_format_is_linked_get_fixture
        assert data_labels.number_format_is_linked is expected_value

    def it_can_change_whether_its_number_format_is_linked(
        self, number_format_is_linked_set_fixture
    ):
        data_labels, new_value, expected_xml = number_format_is_linked_set_fixture
        data_labels.number_format_is_linked = new_value
        assert data_labels._element.xml == expected_xml

    def it_knows_its_position(self, position_get_fixture):
        data_labels, expected_value = position_get_fixture
        assert data_labels.position == expected_value

    def it_can_change_its_position(self, position_set_fixture):
        data_labels, new_value, expected_xml = position_set_fixture
        data_labels.position = new_value
        assert data_labels._element.xml == expected_xml

    def it_knows_whether_it_shows_category_name(self, catname_get_fixture):
        dLbls, expected_value = catname_get_fixture
        data_labels = DataLabels(dLbls)

        show_category_name = data_labels.show_category_name

        assert show_category_name == expected_value

    def it_can_set_whether_it_shows_category_name(self, catname_set_fixture):
        dLbls, new_value, expected_xml = catname_set_fixture
        data_labels = DataLabels(dLbls)

        data_labels.show_category_name = new_value

        assert data_labels._element.xml == expected_xml

    def it_knows_whether_it_shows_legend_key(self, lgndkey_get_fixture):
        dLbls, expected_value = lgndkey_get_fixture
        data_labels = DataLabels(dLbls)

        show_legend_key = data_labels.show_legend_key

        assert show_legend_key == expected_value

    def it_can_set_whether_it_shows_legend_key(self, lgndkey_set_fixture):
        dLbls, new_value, expected_xml = lgndkey_set_fixture
        data_labels = DataLabels(dLbls)

        data_labels.show_legend_key = new_value

        assert data_labels._element.xml == expected_xml

    def it_knows_whether_it_shows_percentage(self, percent_get_fixture):
        dLbls, expected_value = percent_get_fixture
        data_labels = DataLabels(dLbls)

        show_percentage = data_labels.show_percentage

        assert show_percentage == expected_value

    def it_can_set_whether_it_shows_percentage(self, percent_set_fixture):
        dLbls, new_value, expected_xml = percent_set_fixture
        data_labels = DataLabels(dLbls)

        data_labels.show_percentage = new_value

        assert data_labels._element.xml == expected_xml

    def it_knows_whether_it_shows_series_name(self, sername_get_fixture):
        dLbls, expected_value = sername_get_fixture
        data_labels = DataLabels(dLbls)

        show_series_name = data_labels.show_series_name

        assert show_series_name == expected_value

    def it_can_set_whether_it_shows_series_name(self, sername_set_fixture):
        dLbls, new_value, expected_xml = sername_set_fixture
        data_labels = DataLabels(dLbls)

        data_labels.show_series_name = new_value

        assert data_labels._element.xml == expected_xml

    def it_knows_whether_it_shows_value(self, value_get_fixture):
        dLbls, expected_value = value_get_fixture
        data_labels = DataLabels(dLbls)

        show_value = data_labels.show_value

        assert show_value == expected_value

    def it_can_set_whether_it_shows_value(self, value_set_fixture):
        dLbls, new_value, expected_xml = value_set_fixture
        data_labels = DataLabels(dLbls)

        data_labels.show_value = new_value

        assert data_labels._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:dLbls", False),
            ("c:dLbls/c:showCatName{val=0}", False),
            ("c:dLbls/c:showCatName{val=1}", True),
            ("c:dLbls/c:showCatName{val=true}", True),
            # ---not an expected situation, but schema-compliant---
            ("c:dLbls/c:showCatName", True),
        ]
    )
    def catname_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        dLbls = element(dLbls_cxml)
        return dLbls, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", False, "c:dLbls/c:showCatName{val=0}"),
            ("c:dLbls", True, "c:dLbls/c:showCatName{val=1}"),
            ("c:dLbls/c:showCatName", False, "c:dLbls/c:showCatName{val=0}"),
            ("c:dLbls/c:showCatName{val=0}", False, "c:dLbls/c:showCatName{val=0}"),
            ("c:dLbls/c:showCatName{val=0}", True, "c:dLbls/c:showCatName{val=1}"),
            ("c:dLbls/c:showCatName{val=1}", False, "c:dLbls/c:showCatName{val=0}"),
        ]
    )
    def catname_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        dLbls = element(dLbls_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return dLbls, new_value, expected_dLbls_xml

    @pytest.fixture
    def font_fixture(self, Font_, font_):
        dLbls = element("c:dLbls/c:txPr/a:p/a:pPr/a:defRPr")
        defRPr = dLbls.xpath(".//a:defRPr")[0]
        data_labels = DataLabels(dLbls)
        return data_labels, Font_, defRPr, font_

    @pytest.fixture(
        params=[
            ("c:dLbls", False),
            ("c:dLbls/c:showLegendKey{val=0}", False),
            ("c:dLbls/c:showLegendKey{val=1}", True),
            ("c:dLbls/c:showLegendKey{val=true}", True),
            # ---not an expected situation, but schema-compliant---
            ("c:dLbls/c:showLegendKey", True),
        ]
    )
    def lgndkey_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        dLbls = element(dLbls_cxml)
        return dLbls, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", False, "c:dLbls/c:showLegendKey{val=0}"),
            ("c:dLbls", True, "c:dLbls/c:showLegendKey{val=1}"),
            ("c:dLbls/c:showLegendKey", False, "c:dLbls/c:showLegendKey{val=0}"),
            ("c:dLbls/c:showLegendKey{val=0}", False, "c:dLbls/c:showLegendKey{val=0}"),
            ("c:dLbls/c:showLegendKey{val=0}", True, "c:dLbls/c:showLegendKey{val=1}"),
            ("c:dLbls/c:showLegendKey{val=1}", False, "c:dLbls/c:showLegendKey{val=0}"),
        ]
    )
    def lgndkey_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        dLbls = element(dLbls_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return dLbls, new_value, expected_dLbls_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", "General"),
            ("c:dLbls/c:numFmt{formatCode=foobar}", "foobar"),
        ]
    )
    def number_format_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        return data_labels, expected_value

    @pytest.fixture(
        params=[
            (
                "c:dLbls",
                "General",
                "c:dLbls/c:numFmt{formatCode=General,sourceLinked=0}",
            ),
            (
                "c:dLbls/c:numFmt{formatCode=General}",
                "00.00",
                "c:dLbls/c:numFmt{formatCode=00.00,sourceLinked=0}",
            ),
        ]
    )
    def number_format_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        expected_xml = xml(expected_dLbls_cxml)
        return data_labels, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", True),
            ("c:dLbls/c:numFmt", True),
            ("c:dLbls/c:numFmt{sourceLinked=0}", False),
            ("c:dLbls/c:numFmt{sourceLinked=1}", True),
        ]
    )
    def number_format_is_linked_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        return data_labels, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", True, "c:dLbls/c:numFmt{sourceLinked=1}"),
            ("c:dLbls", False, "c:dLbls/c:numFmt{sourceLinked=0}"),
            ("c:dLbls", None, "c:dLbls/c:numFmt"),
            ("c:dLbls/c:numFmt", True, "c:dLbls/c:numFmt{sourceLinked=1}"),
            (
                "c:dLbls/c:numFmt{sourceLinked=1}",
                False,
                "c:dLbls/c:numFmt{sourceLinked=0}",
            ),
        ]
    )
    def number_format_is_linked_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        expected_xml = xml(expected_dLbls_cxml)
        return data_labels, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", False),
            ("c:dLbls/c:showPercent{val=0}", False),
            ("c:dLbls/c:showPercent{val=1}", True),
            ("c:dLbls/c:showPercent{val=true}", True),
            ("c:dLbls/c:showPercent", True),
        ]
    )
    def percent_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        dLbls = element(dLbls_cxml)
        return dLbls, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", False, "c:dLbls/c:showPercent{val=0}"),
            ("c:dLbls", True, "c:dLbls/c:showPercent{val=1}"),
            ("c:dLbls/c:showPercent", False, "c:dLbls/c:showPercent{val=0}"),
            ("c:dLbls/c:showPercent{val=0}", False, "c:dLbls/c:showPercent{val=0}"),
            ("c:dLbls/c:showPercent{val=0}", True, "c:dLbls/c:showPercent{val=1}"),
            ("c:dLbls/c:showPercent{val=1}", False, "c:dLbls/c:showPercent{val=0}"),
        ]
    )
    def percent_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        dLbls = element(dLbls_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return dLbls, new_value, expected_dLbls_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", None),
            ("c:dLbls/c:dLblPos{val=inBase}", XL_LABEL_POSITION.INSIDE_BASE),
        ]
    )
    def position_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        return data_labels, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", XL_LABEL_POSITION.INSIDE_BASE, "c:dLbls/c:dLblPos{val=inBase}"),
            (
                "c:dLbls/c:dLblPos{val=inBase}",
                XL_LABEL_POSITION.OUTSIDE_END,
                "c:dLbls/c:dLblPos{val=outEnd}",
            ),
            ("c:dLbls/c:dLblPos{val=inBase}", None, "c:dLbls"),
            ("c:dLbls", None, "c:dLbls"),
        ]
    )
    def position_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        expected_xml = xml(expected_dLbls_cxml)
        return data_labels, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", False),
            ("c:dLbls/c:showSerName{val=0}", False),
            ("c:dLbls/c:showSerName{val=1}", True),
            ("c:dLbls/c:showSerName{val=true}", True),
            ("c:dLbls/c:showSerName", True),
        ]
    )
    def sername_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        dLbls = element(dLbls_cxml)
        return dLbls, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", False, "c:dLbls/c:showSerName{val=0}"),
            ("c:dLbls", True, "c:dLbls/c:showSerName{val=1}"),
            ("c:dLbls/c:showSerName", False, "c:dLbls/c:showSerName{val=0}"),
            ("c:dLbls/c:showSerName{val=0}", False, "c:dLbls/c:showSerName{val=0}"),
            ("c:dLbls/c:showSerName{val=0}", True, "c:dLbls/c:showSerName{val=1}"),
            ("c:dLbls/c:showSerName{val=1}", False, "c:dLbls/c:showSerName{val=0}"),
        ]
    )
    def sername_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        dLbls = element(dLbls_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return dLbls, new_value, expected_dLbls_xml

    @pytest.fixture(
        params=[
            (
                "c:dLbls{a:b=c}",
                "c:dLbls{a:b=c}/c:txPr/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr)",
            ),
            (
                "c:dLbls{a:b=c}/c:txPr/(a:bodyPr,a:p)",
                "c:dLbls{a:b=c}/c:txPr/(a:bodyPr,a:p/a:pPr/a:defRPr)",
            ),
            (
                "c:dLbls{a:b=c}/c:txPr/(a:bodyPr,a:p/a:pPr)",
                "c:dLbls{a:b=c}/c:txPr/(a:bodyPr,a:p/a:pPr/a:defRPr)",
            ),
        ]
    )
    def txPr_fixture(self, request):
        dLbls_cxml, expected_cxml = request.param
        data_labels = DataLabels(element(dLbls_cxml))
        expected_xml = xml(expected_cxml)
        return data_labels, expected_xml

    @pytest.fixture(
        params=[
            ("c:dLbls", False),
            ("c:dLbls/c:showVal{val=0}", False),
            ("c:dLbls/c:showVal{val=1}", True),
            ("c:dLbls/c:showVal{val=true}", True),
            ("c:dLbls/c:showVal", True),
        ]
    )
    def value_get_fixture(self, request):
        dLbls_cxml, expected_value = request.param
        dLbls = element(dLbls_cxml)
        return dLbls, expected_value

    @pytest.fixture(
        params=[
            ("c:dLbls", False, "c:dLbls/c:showVal{val=0}"),
            ("c:dLbls", True, "c:dLbls/c:showVal{val=1}"),
            ("c:dLbls/c:showVal", False, "c:dLbls/c:showVal{val=0}"),
            ("c:dLbls/c:showVal{val=0}", False, "c:dLbls/c:showVal{val=0}"),
            ("c:dLbls/c:showVal{val=0}", True, "c:dLbls/c:showVal{val=1}"),
            ("c:dLbls/c:showVal{val=1}", False, "c:dLbls/c:showVal{val=0}"),
        ]
    )
    def value_set_fixture(self, request):
        dLbls_cxml, new_value, expected_dLbls_cxml = request.param
        dLbls = element(dLbls_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return dLbls, new_value, expected_dLbls_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Font_(self, request, font_):
        return class_mock(request, "pptx.chart.datalabel.Font", return_value=font_)

    @pytest.fixture
    def font_(self, request):
        return instance_mock(request, Font)



# encoding: utf-8

"""
ChartData and related objects.
"""

from __future__ import absolute_import, print_function, unicode_literals

import datetime
from numbers import Number

from pptx.chart.xlsx import (
    BubbleWorkbookWriter,
    CategoryWorkbookWriter,
    XyWorkbookWriter,
)
from pptx.chart.xmlwriter import ChartXmlWriter
from pptx.compat import Sequence
from pptx.util import lazyproperty


class _BaseChartData(Sequence):
    """
    Base class providing common members for chart data objects. A chart data
    object serves as a proxy for the chart data table that will be written to
    an Excel worksheet; operating as a sequence of series as well as
    providing access to chart-level attributes. A chart data object is used
    as a parameter in :meth:`shapes.add_chart` and
    :meth:`Chart.replace_data`. The data structure varies between major chart
    categories such as category charts and XY charts.
    """

    def __init__(self, number_format="General"):
        super(_BaseChartData, self).__init__()
        self._number_format = number_format
        self._series = []

    def __getitem__(self, index):
        return self._series.__getitem__(index)

    def __len__(self):
        return self._series.__len__()

    def append(self, series):
        return self._series.append(series)

    def data_point_offset(self, series):
        """
        The total integer number of data points appearing in the series of
        this chart that are prior to *series* in this sequence.
        """
        count = 0
        for this_series in self:
            if series is this_series:
                return count
            count += len(this_series)
        raise ValueError("series not in chart data object")

    @property
    def number_format(self):
        """
        The formatting template string, e.g. '#,##0.0', that determines how
        X and Y values are formatted in this chart and in the Excel
        spreadsheet. A number format specified on a series will override this
        value for that series. Likewise, a distinct number format can be
        specified for a particular data point within a series.
        """
        return self._number_format

    def series_index(self, series):
        """
        Return the integer index of *series* in this sequence.
        """
        for idx, s in enumerate(self):
            if series is s:
                return idx
        raise ValueError("series not in chart data object")

    def series_name_ref(self, series):
        """
        Return the Excel worksheet reference to the cell containing the name
        for *series*.
        """
        return self._workbook_writer.series_name_ref(series)

    def x_values_ref(self, series):
        """
        The Excel worksheet reference to the X values for *series* (not
        including the column label).
        """
        return self._workbook_writer.x_values_ref(series)

    @property
    def xlsx_blob(self):
        """
        Return a blob containing an Excel workbook file populated with the
        contents of this chart data object.
        """
        return self._workbook_writer.xlsx_blob

    def xml_bytes(self, chart_type):
        """
        Return a blob containing the XML for a chart of *chart_type*
        containing the series in this chart data object, as bytes suitable
        for writing directly to a file.
        """
        return self._xml(chart_type).encode("utf-8")

    def y_values_ref(self, series):
        """
        The Excel worksheet reference to the Y values for *series* (not
        including the column label).
        """
        return self._workbook_writer.y_values_ref(series)

    @property
    def _workbook_writer(self):
        """
        The worksheet writer object to which layout and writing of the Excel
        worksheet for this chart will be delegated.
        """
        raise NotImplementedError("must be implemented by all subclasses")

    def _xml(self, chart_type):
        """
        Return (as unicode text) the XML for a chart of *chart_type*
        populated with the values in this chart data object. The XML is
        a complete XML document, including an XML declaration specifying
        UTF-8 encoding.
        """
        return ChartXmlWriter(chart_type, self).xml


class _BaseSeriesData(Sequence):
    """
    Base class providing common members for series data objects. A series
    data object serves as proxy for a series data column in the Excel
    worksheet. It operates as a sequence of data points, as well as providing
    access to series-level attributes like the series label.
    """

    def __init__(self, chart_data, name, number_format):
        self._chart_data = chart_data
        self._name = name
        self._number_format = number_format
        self._data_points = []

    def __getitem__(self, index):
        return self._data_points.__getitem__(index)

    def __len__(self):
        return self._data_points.__len__()

    def append(self, data_point):
        return self._data_points.append(data_point)

    @property
    def data_point_offset(self):
        """
        The integer count of data points that appear in all chart series
        prior to this one.
        """
        return self._chart_data.data_point_offset(self)

    @property
    def index(self):
        """
        Zero-based integer indicating the sequence position of this series in
        its chart. For example, the second of three series would return `1`.
        """
        return self._chart_data.series_index(self)

    @property
    def name(self):
        """
        The name of this series, e.g. 'Series 1'. This name is used as the
        column heading for the y-values of this series and may also appear in
        the chart legend and perhaps other chart locations.
        """
        return self._name if self._name is not None else ""

    @property
    def name_ref(self):
        """
        The Excel worksheet reference to the cell containing the name for
        this series.
        """
        return self._chart_data.series_name_ref(self)

    @property
    def number_format(self):
        """
        The formatting template string that determines how a number in this
        series is formatted, both in the chart and in the Excel spreadsheet;
        for example '#,##0.0'. If not specified for this series, it is
        inherited from the parent chart data object.
        """
        number_format = self._number_format
        if number_format is None:
            return self._chart_data.number_format
        return number_format

    @property
    def x_values(self):
        """
        A sequence containing the X value of each datapoint in this series,
        in data point order.
        """
        return [dp.x for dp in self._data_points]

    @property
    def x_values_ref(self):
        """
        The Excel worksheet reference to the X values for this chart (not
        including the column heading).
        """
        return self._chart_data.x_values_ref(self)

    @property
    def y_values(self):
        """
        A sequence containing the Y value of each datapoint in this series,
        in data point order.
        """
        return [dp.y for dp in self._data_points]

    @property
    def y_values_ref(self):
        """
        The Excel worksheet reference to the Y values for this chart (not
        including the column heading).
        """
        return self._chart_data.y_values_ref(self)


class _BaseDataPoint(object):
    """
    Base class providing common members for data point objects.
    """

    def __init__(self, series_data, number_format):
        super(_BaseDataPoint, self).__init__()
        self._series_data = series_data
        self._number_format = number_format

    @property
    def number_format(self):
        """
        The formatting template string that determines how the value of this
        data point is formatted, both in the chart and in the Excel
        spreadsheet; for example '#,##0.0'. If not specified for this data
        point, it is inherited from the parent series data object.
        """
        number_format = self._number_format
        if number_format is None:
            return self._series_data.number_format
        return number_format


class CategoryChartData(_BaseChartData):
    """
    Accumulates data specifying the categories and series values for a chart
    and acts as a proxy for the chart data table that will be written to an
    Excel worksheet. Used as a parameter in :meth:`shapes.add_chart` and
    :meth:`Chart.replace_data`.

    This object is suitable for use with category charts, i.e. all those
    having a discrete set of label values (categories) as the range of their
    independent variable (X-axis) values. Unlike the ChartData types for
    charts supporting a continuous range of independent variable values (such
    as XyChartData), CategoryChartData has a single collection of category
    (X) values and each data point in its series specifies only the Y value.
    The corresponding X value is inferred by its position in the sequence.
    """

    def add_category(self, label):
        """
        Return a newly created |data.Category| object having *label* and
        appended to the end of the category collection for this chart.
        *label* can be a string, a number, a datetime.date, or
        datetime.datetime object. All category labels in a chart must be the
        same type. All category labels in a chart having multi-level
        categories must be strings.
        """
        return self.categories.add_category(label)

    def add_series(self, name, values=(), number_format=None):
        """
        Add a series to this data set entitled *name* and having the data
        points specified by *values*, an iterable of numeric values.
        *number_format* specifies how the series values will be displayed,
        and may be a string, e.g. '#,##0' corresponding to an Excel number
        format.
        """
        series_data = CategorySeriesData(self, name, number_format)
        self.append(series_data)
        for value in values:
            series_data.add_data_point(value)
        return series_data

    @lazyproperty
    def categories(self):
        """
        A |data.Categories| object providing access to the hierarchy of
        category objects for this chart data. Assigning an iterable of
        category labels (strings, numbers, or dates) replaces the
        |data.Categories| object with a new one containing a category for
        each label in the sequence.

        Creating a chart from chart data having date categories will cause
        the chart to have a |DateAxis| for its category axis.
        """
        return Categories()

    @categories.setter
    def categories(self, category_labels):
        categories = Categories()
        for label in category_labels:
            categories.add_category(label)
        self._categories = categories

    @property
    def categories_ref(self):
        """
        The Excel worksheet reference to the categories for this chart (not
        including the column heading).
        """
        return self._workbook_writer.categories_ref

    def values_ref(self, series):
        """
        The Excel worksheet reference to the values for *series* (not
        including the column heading).
        """
        return self._workbook_writer.values_ref(series)

    @lazyproperty
    def _workbook_writer(self):
        """
        The worksheet writer object to which layout and writing of the Excel
        worksheet for this chart will be delegated.
        """
        return CategoryWorkbookWriter(self)


class Categories(Sequence):
    """
    A sequence of |data.Category| objects, also having certain hierarchical
    graph behaviors for support of multi-level (nested) categories.
    """

    def __init__(self):
        super(Categories, self).__init__()
        self._categories = []
        self._number_format = None

    def __getitem__(self, idx):
        return self._categories.__getitem__(idx)

    def __len__(self):
        """
        Return the count of the highest level of category in this sequence.
        If it contains hierarchical (multi-level) categories, this number
        will differ from :attr:`category_count`, which is the number of leaf
        nodes.
        """
        return self._categories.__len__()

    def add_category(self, label):
        """
        Return a newly created |data.Category| object having *label* and
        appended to the end of this category sequence. *label* can be
        a string, a number, a datetime.date, or datetime.datetime object. All
        category labels in a chart must be the same type. All category labels
        in a chart having multi-level categories must be strings.

        Creating a chart from chart data having date categories will cause
        the chart to have a |DateAxis| for its category axis.
        """
        category = Category(label, self)
        self._categories.append(category)
        return category

    @property
    def are_dates(self):
        """
        Return |True| if the first category in this collection has a date
        label (as opposed to str or numeric). A date label is one of type
        datetime.date or datetime.datetime. Returns |False| otherwise,
        including when this category collection is empty. It also returns
        False when this category collection is hierarchical, because
        hierarchical categories can only be written as string labels.
        """
        if self.depth != 1:
            return False
        first_cat_label = self[0].label
        date_types = (datetime.date, datetime.datetime)
        if isinstance(first_cat_label, date_types):
            return True
        return False

    @property
    def are_numeric(self):
        """
        Return |True| if the first category in this collection has a numeric
        label (as opposed to a string label), including if that value is
        a datetime.date or datetime.datetime object (as those are converted
        to integers for storage in Excel). Returns |False| otherwise,
        including when this category collection is empty. It also returns
        False when this category collection is hierarchical, because
        hierarchical categories can only be written as string labels.
        """
        if self.depth != 1:
            return False
        # This method only tests the first category. The categories must
        # be of uniform type, and if they're not, there will be problems
        # later in the process, but it's not this method's job to validate
        # the caller's input.
        first_cat_label = self[0].label
        numeric_types = (Number, datetime.date, datetime.datetime)
        if isinstance(first_cat_label, numeric_types):
            return True
        return False

    @property
    def depth(self):
        """
        The number of hierarchy levels in this category graph. Returns 0 if
        it contains no categories.
        """
        categories = self._categories
        if not categories:
            return 0
        first_depth = categories[0].depth
        for category in categories[1:]:
            if category.depth != first_depth:
                raise ValueError("category depth not uniform")
        return first_depth

    def index(self, category):
        """
        The offset of *category* in the overall sequence of leaf categories.
        A non-leaf category gets the index of its first sub-category.
        """
        index = 0
        for this_category in self._categories:
            if category is this_category:
                return index
            index += this_category.leaf_count
        raise ValueError("category not in top-level categories")

    @property
    def leaf_count(self):
        """
        The number of leaf-level categories in this hierarchy. The return
        value is the same as that of `len()` only when the hierarchy is
        single level.
        """
        return sum(c.leaf_count for c in self._categories)

    @property
    def levels(self):
        """
        A generator of (idx, label) sequences representing the category
        hierarchy from the bottom up. The first level contains all leaf
        categories, and each subsequent is the next level up.
        """

        def levels(categories):
            # yield all lower levels
            sub_categories = [sc for c in categories for sc in c.sub_categories]
            if sub_categories:
                for level in levels(sub_categories):
                    yield level
            # yield this level
            yield [(cat.idx, cat.label) for cat in categories]

        for level in levels(self):
            yield level

    @property
    def number_format(self):
        """
        Read/write. Return a string representing the number format used in
        Excel to format these category values, e.g. '0.0' or 'mm/dd/yyyy'.
        This string is only relevant when the categories are numeric or date
        type, although it returns 'General' without error when the categories
        are string labels. Assigning |None| causes the default number format
        to be used, based on the type of the category labels.
        """
        GENERAL = "General"

        # defined value takes precedence
        if self._number_format is not None:
            return self._number_format

        # multi-level (should) always be string labels
        # zero depth means empty in which case we can't tell anyway
        if self.depth != 1:
            return GENERAL

        # everything except dates gets 'General'
        first_cat_label = self[0].label
        if isinstance(first_cat_label, (datetime.date, datetime.datetime)):
            return r"yyyy\-mm\-dd"
        return GENERAL

    @number_format.setter
    def number_format(self, value):
        self._number_format = value


class Category(object):
    """
    A chart category, primarily having a label to be displayed on the
    category axis, but also able to be configured in a hierarchy for support
    of multi-level category charts.
    """

    def __init__(self, label, parent):
        super(Category, self).__init__()
        self._label = label
        self._parent = parent
        self._sub_categories = []

    def add_sub_category(self, label):
        """
        Return a newly created |data.Category| object having *label* and
        appended to the end of the sub-category sequence for this category.
        """
        category = Category(label, self)
        self._sub_categories.append(category)
        return category

    @property
    def depth(self):
        """
        The number of hierarchy levels rooted at this category node. Returns
        1 if this category has no sub-categories.
        """
        sub_categories = self._sub_categories
        if not sub_categories:
            return 1
        first_depth = sub_categories[0].depth
        for category in sub_categories[1:]:
            if category.depth != first_depth:
                raise ValueError("category depth not uniform")
        return first_depth + 1

    @property
    def idx(self):
        """
        The offset of this category in the overall sequence of leaf
        categories. A non-leaf category gets the index of its first
        sub-category.
        """
        return self._parent.index(self)

    def index(self, sub_category):
        """
        The offset of *sub_category* in the overall sequence of leaf
        categories.
        """
        index = self._parent.index(self)
        for this_sub_category in self._sub_categories:
            if sub_category is this_sub_category:
                return index
            index += this_sub_category.leaf_count
        raise ValueError("sub_category not in this category")

    @property
    def leaf_count(self):
        """
        The number of leaf category nodes under this category. Returns
        1 if this category has no sub-categories.
        """
        if not self._sub_categories:
            return 1
        return sum(category.leaf_count for category in self._sub_categories)

    @property
    def label(self):
        """
        The value that appears on the axis for this category. The label can
        be a string, a number, or a datetime.date or datetime.datetime
        object.
        """
        return self._label if self._label is not None else ""

    def numeric_str_val(self, date_1904=False):
        """
        The string representation of the numeric (or date) label of this
        category, suitable for use in the XML `c:pt` element for this
        category. The optional *date_1904* parameter specifies the epoch used
        for calculating Excel date numbers.
        """
        label = self._label
        if isinstance(label, (datetime.date, datetime.datetime)):
            return "%.1f" % self._excel_date_number(date_1904)
        return str(self._label)

    @property
    def sub_categories(self):
        """
        The sequence of child categories for this category.
        """
        return self._sub_categories

    def _excel_date_number(self, date_1904):
        """
        Return an integer representing the date label of this category as the
        number of days since January 1, 1900 (or 1904 if date_1904 is
        |True|).
        """
        date, label = datetime.date, self._label
        # -- get date from label in type-independent-ish way
        date_ = date(label.year, label.month, label.day)
        epoch = date(1904, 1, 1) if date_1904 else date(1899, 12, 31)
        delta = date_ - epoch
        excel_day_number = delta.days

        # -- adjust for Excel mistaking 1900 for a leap year --
        if not date_1904 and excel_day_number > 59:
            excel_day_number += 1

        return excel_day_number


class ChartData(CategoryChartData):
    """
    |ChartData| is simply an alias for |CategoryChartData| and may be removed
    in a future release. All new development should use |CategoryChartData|
    for creating or replacing the data in chart types other than XY and
    Bubble.
    """


class CategorySeriesData(_BaseSeriesData):
    """
    The data specific to a particular category chart series. It provides
    access to the series label, the series data points, and an optional
    number format to be applied to each data point not having a specified
    number format.
    """

    def add_data_point(self, value, number_format=None):
        """
        Return a CategoryDataPoint object newly created with value *value*,
        an optional *number_format*, and appended to this sequence.
        """
        data_point = CategoryDataPoint(self, value, number_format)
        self.append(data_point)
        return data_point

    @property
    def categories(self):
        """
        The |data.Categories| object that provides access to the category
        objects for this series.
        """
        return self._chart_data.categories

    @property
    def categories_ref(self):
        """
        The Excel worksheet reference to the categories for this chart (not
        including the column heading).
        """
        return self._chart_data.categories_ref

    @property
    def values(self):
        """
        A sequence containing the (Y) value of each datapoint in this series,
        in data point order.
        """
        return [dp.value for dp in self._data_points]

    @property
    def values_ref(self):
        """
        The Excel worksheet reference to the (Y) values for this series (not
        including the column heading).
        """
        return self._chart_data.values_ref(self)


class XyChartData(_BaseChartData):
    """
    A specialized ChartData object suitable for use with an XY (aka. scatter)
    chart. Unlike ChartData, it has no category sequence. Rather, each data
    point of each series specifies both an X and a Y value.
    """

    def add_series(self, name, number_format=None):
        """
        Return an |XySeriesData| object newly created and added at the end of
        this sequence, identified by *name* and values formatted with
        *number_format*.
        """
        series_data = XySeriesData(self, name, number_format)
        self.append(series_data)
        return series_data

    @lazyproperty
    def _workbook_writer(self):
        """
        The worksheet writer object to which layout and writing of the Excel
        worksheet for this chart will be delegated.
        """
        return XyWorkbookWriter(self)


class BubbleChartData(XyChartData):
    """
    A specialized ChartData object suitable for use with a bubble chart.
    A bubble chart is essentially an XY chart where the markers are scaled to
    provide a third quantitative dimension to the exhibit.
    """

    def add_series(self, name, number_format=None):
        """
        Return a |BubbleSeriesData| object newly created and added at the end
        of this sequence, and having series named *name* and values formatted
        with *number_format*.
        """
        series_data = BubbleSeriesData(self, name, number_format)
        self.append(series_data)
        return series_data

    def bubble_sizes_ref(self, series):
        """
        The Excel worksheet reference for the range containing the bubble
        sizes for *series*.
        """
        return self._workbook_writer.bubble_sizes_ref(series)

    @lazyproperty
    def _workbook_writer(self):
        """
        The worksheet writer object to which layout and writing of the Excel
        worksheet for this chart will be delegated.
        """
        return BubbleWorkbookWriter(self)


class XySeriesData(_BaseSeriesData):
    """
    The data specific to a particular XY chart series. It provides access to
    the series label, the series data points, and an optional number format
    to be applied to each data point not having a specified number format.

    The sequence of data points in an XY series is significant; lines are
    plotted following the sequence of points, even if that causes a line
    segment to "travel backward" (implying a multi-valued function). The data
    points are not automatically sorted into increasing order by X value.
    """

    def add_data_point(self, x, y, number_format=None):
        """
        Return an XyDataPoint object newly created with values *x* and *y*,
        and appended to this sequence.
        """
        data_point = XyDataPoint(self, x, y, number_format)
        self.append(data_point)
        return data_point


class BubbleSeriesData(XySeriesData):
    """
    The data specific to a particular Bubble chart series. It provides access
    to the series label, the series data points, and an optional number
    format to be applied to each data point not having a specified number
    format.

    The sequence of data points in a bubble chart series is maintained
    throughout the chart building process because a data point has no unique
    identifier and can only be retrieved by index.
    """

    def add_data_point(self, x, y, size, number_format=None):
        """
        Append a new BubbleDataPoint object having the values *x*, *y*, and
        *size*. The optional *number_format* is used to format the Y value.
        If not provided, the number format is inherited from the series data.
        """
        data_point = BubbleDataPoint(self, x, y, size, number_format)
        self.append(data_point)
        return data_point

    @property
    def bubble_sizes(self):
        """
        A sequence containing the bubble size for each datapoint in this
        series, in data point order.
        """
        return [dp.bubble_size for dp in self._data_points]

    @property
    def bubble_sizes_ref(self):
        """
        The Excel worksheet reference for the range containing the bubble
        sizes for this series.
        """
        return self._chart_data.bubble_sizes_ref(self)


class CategoryDataPoint(_BaseDataPoint):
    """
    A data point in a category chart series. Provides access to the value of
    the datapoint and the number format with which it should appear in the
    Excel file.
    """

    def __init__(self, series_data, value, number_format):
        super(CategoryDataPoint, self).__init__(series_data, number_format)
        self._value = value

    @property
    def value(self):
        """
        The (Y) value for this category data point.
        """
        return self._value


class XyDataPoint(_BaseDataPoint):
    """
    A data point in an XY chart series. Provides access to the x and y values
    of the datapoint.
    """

    def __init__(self, series_data, x, y, number_format):
        super(XyDataPoint, self).__init__(series_data, number_format)
        self._x = x
        self._y = y

    @property
    def x(self):
        """
        The X value for this XY data point.
        """
        return self._x

    @property
    def y(self):
        """
        The Y value for this XY data point.
        """
        return self._y


class BubbleDataPoint(XyDataPoint):
    """
    A data point in a bubble chart series. Provides access to the x, y, and
    size values of the datapoint.
    """

    def __init__(self, series_data, x, y, size, number_format):
        super(BubbleDataPoint, self).__init__(series_data, x, y, number_format)
        self._size = size

    @property
    def bubble_size(self):
        """
        The value representing the size of the bubble for this data point.
        """
        return self._size



# encoding: utf-8

"""Provides Python 2 compatibility objects."""

from StringIO import StringIO as BytesIO  # noqa


def is_integer(obj):
    """Return True if *obj* is an integer (int, long), False otherwise."""
    return isinstance(obj, (int, long))  # noqa F821


def is_string(obj):
    """Return True if *obj* is a string, False otherwise."""
    return isinstance(obj, basestring)  # noqa F821


def is_unicode(obj):
    """Return True if *obj* is a unicode string, False otherwise."""
    return isinstance(obj, unicode)  # noqa F821


def to_unicode(text):
    """Return *text* as a unicode string.

    *text* can be a 7-bit ASCII string, a UTF-8 encoded 8-bit string, or unicode. String
    values are converted to unicode assuming UTF-8 encoding. Unicode values are returned
    unchanged.
    """
    # both str and unicode inherit from basestring
    if not isinstance(text, basestring):  # noqa F821
        tmpl = "expected unicode or UTF-8 (or ASCII) encoded str, got %s value %s"
        raise TypeError(tmpl % (type(text), text))
    # return unicode strings unchanged
    if isinstance(text, unicode):  # noqa F821
        return text
    # otherwise assume UTF-8 encoding, which also works for ASCII
    return unicode(text, "utf-8")  # noqa F821


Unicode = unicode  # noqa F821



# encoding: utf-8

"""
Unit test suite for the pptx.dml.chtfmt module.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.chtfmt import ChartFormat
from pptx.dml.fill import FillFormat
from pptx.dml.line import LineFormat

from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock


class DescribeChartFormat(object):
    def it_provides_access_to_its_fill(self, fill_fixture):
        chart_format, FillFormat_, fill_, expected_xml = fill_fixture
        fill = chart_format.fill
        FillFormat_.from_fill_parent.assert_called_once_with(
            chart_format._element.xpath("c:spPr")[0]
        )
        assert fill is fill_
        assert chart_format._element.xml == expected_xml

    def it_provides_access_to_its_line(self, line_fixture):
        chart_format, LineFormat_, line_, expected_xml = line_fixture
        line = chart_format.line
        LineFormat_.assert_called_once_with(chart_format._element.xpath("c:spPr")[0])
        assert line is line_
        assert chart_format._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:catAx", "c:catAx/c:spPr"),
            ("c:catAx/c:spPr", "c:catAx/c:spPr"),
            ("c:dPt", "c:dPt/c:spPr"),
            ("c:dPt/c:spPr", "c:dPt/c:spPr"),
            ("c:majorGridlines", "c:majorGridlines/c:spPr"),
            ("c:majorGridlines/c:spPr", "c:majorGridlines/c:spPr"),
            ("c:valAx", "c:valAx/c:spPr"),
            ("c:valAx/c:spPr", "c:valAx/c:spPr"),
        ]
    )
    def fill_fixture(self, request, FillFormat_, fill_):
        dPt_cxml, expected_cxml = request.param
        chart_format = ChartFormat(element(dPt_cxml))
        FillFormat_.from_fill_parent.return_value = fill_
        expected_xml = xml(expected_cxml)
        return chart_format, FillFormat_, fill_, expected_xml

    @pytest.fixture(
        params=[
            ("c:catAx", "c:catAx/c:spPr"),
            ("c:catAx/c:spPr", "c:catAx/c:spPr"),
            ("c:dPt", "c:dPt/c:spPr"),
            ("c:dPt/c:spPr", "c:dPt/c:spPr"),
            ("c:majorGridlines", "c:majorGridlines/c:spPr"),
            ("c:majorGridlines/c:spPr", "c:majorGridlines/c:spPr"),
            ("c:valAx", "c:valAx/c:spPr"),
            ("c:valAx/c:spPr", "c:valAx/c:spPr"),
        ]
    )
    def line_fixture(self, request, LineFormat_, line_):
        cxml, expected_cxml = request.param
        chart_format = ChartFormat(element(cxml))
        expected_xml = xml(expected_cxml)
        return chart_format, LineFormat_, line_, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def FillFormat_(self, request):
        return class_mock(request, "pptx.dml.chtfmt.FillFormat")

    @pytest.fixture
    def fill_(self, request):
        return instance_mock(request, FillFormat)

    @pytest.fixture
    def LineFormat_(self, request, line_):
        return class_mock(request, "pptx.dml.chtfmt.LineFormat", return_value=line_)

    @pytest.fixture
    def line_(self, request):
        return instance_mock(request, LineFormat)



# -*- coding: utf-8 -*-
#
# environment.py
#
# Copyright (C) 2013 Steve Canny scanny@cisco.com
#
# This module is part of python-pptx and is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.php

"""
Used by behave to set testing environment before and after running acceptance
tests.
"""

import os

scratch_dir = os.path.abspath(
    os.path.join(os.path.split(__file__)[0], '_scratch')
)


def before_all(context):
    if not os.path.isdir(scratch_dir):
        os.mkdir(scratch_dir)
